#! /usr/bin/python
# -*- coding: utf-8 -*-
"""
Module for experiment support
"""

import argparse
import logging
import os

logger = logging.getLogger(__name__)

import matplotlib.pyplot as plt
import subprocess
import glob
import numpy as np
import copy
# import six
import pandas
import traceback
import shutil

# ----------------- my scripts --------
import misc
import volumetry_evaluation
import io3d
import io3d.misc
# <codecell>


class RunAndMakeReport:
    def __init__(self, experiment_dir, labels, sliver_reference_dir,
                 input_data_path_pattern, conf_default=None,
                 conf_list=None, show=True, use_plt=True,
                 image_basename=None, only_if_necessary=True,
                 pklz_dirs=None, experiment_name=None,
                 filename_separator='-', markers=None,
                 clean_experiment_dir=False
                 ):

        """
        Je potřeba mít připraven adresář s daty(pklz_dirs), v nichž jsou uloženy
        seedy. Dále pak soubory s konfigurací a stejným jménem jako adresář +
        přípona '.conf'.

        :param experiment_dir: base directory of experiment outputs
        :params labels: Labels for describing graphs. They are also used for
            generating dir names if pklz_dirs is not used.
        :param sliver_reference_dir: dir with sliver reference data
        :param input_data_path_pattern: Directory containing data with seeds. For
            example::

            "/home/mjirik/exp010-seeds/*-seeds.pklz"
        :param conf_default: default values for config generation.
            Example::

            conf_default = {
                'config_version': [1,0,0],
                'working_voxelsize_mm': 2.0,
                'segparams': {
                    'pairwise_alpha_per_mm2': 50,
                    'return_only_object_with_seeds': True,
                    'method': 'GC'
                }
            }
        :param experiment_name: string is used as prefix for all output files
        :param pklz_dirs: List of dirs or string with dir prefix. If the string is
            given base on labels the list is generated
            In each will be output for one experiment. There
            is also required to have file with same name as directory with
            extension.config. In this files is configuration of Lisa.
        :param image_basename: Basis for image names. It can be directory, or
            filename begining
        :param only_if_necessary: If it is True the make_all() function call
            run_experiments() and evaluation() function only if there are no
            evaluation outputs and experiment_outputs.
        :param markers: Allows to control markes for output graphs

        self.dataframe_all is storage for all outpt mesurements
        """

        self.conf_list = conf_list
        self.conf_default = conf_default
        self.labels = labels
        self.sliver_dir = sliver_reference_dir
        self.experiment_dir = experiment_dir
        self.df_all = None
        if experiment_name is None:
            aa, experiment_name = os.path.split(self.experiment_dir)
        self.experiment_name = experiment_name

        if clean_experiment_dir:
            if os.path.exists(experiment_dir):
                shutil.rmtree(experiment_dir)
            os.mkdir(experiment_dir)

        # if isinstance(pklz_dirs, six.string_types):
        if pklz_dirs is None:
            # pklz_dir is string
            pklz_dirs = [
                os.path.join(
                    experiment_dir,
                    experiment_name + filename_separator + lab.replace(' ', ''))
                for lab in labels
            ]
        self.pklz_dirs = pklz_dirs
        self.yaml_files = \
            [os.path.normpath(path) + '.yaml' for path in pklz_dirs]
        self.eval_files = \
            [os.path.normpath(path) + '_eval' for path in pklz_dirs]
        self.exp_conf_files = \
            [os.path.normpath(path) + '.config' for path in pklz_dirs]
        if image_basename is None:
            # self.image_basename, head = os.path.split(self.pklz_dirs[0])
            self.image_basename = os.path.join(
                self.experiment_dir, experiment_name + filename_separator)

        self.show = show
        self.input_data_path_pattern = input_data_path_pattern
        self.use_plt = use_plt
        self.only_if_necessary = only_if_necessary
        self.filename_separator = filename_separator
        if markers is None:
            markers = [
                "ks", "k<", "k>", "k^", "kv", "kp", 'k*', 'ok', 'kh', 'ks'
            ]
        # repeat markers to have requested length
        n = len(labels)
        markers = np.asarray(([markers] * ((n / len(markers) + 1)))
                             ).reshape(-1)[:n].tolist()
        self.markers = markers
        self.dataframe_all = None

    def make_all(self):
        """
        Run config(), run_experiments(), evaluation() and report()
        """
        self.config()
        if self.only_if_necessary:
            if self.is_evaluation_necessary():
                if self.is_run_experiments_necessary():
                    self.run_experiments()
                self.evaluation()
        else:
            self.run_experiments()
            self.evaluation()
        self.report()

    def config(self):
        if self.conf_list is not None:
            logger.info("generate configuration")
            generate_configs(self.pklz_dirs, self.conf_default, self.conf_list)

    def run_experiments(self):
        logger.info("run experiments")
        run_all_liver_segmentation_experiments_with_conf(
            self.exp_conf_files,
            self.input_data_path_pattern,
            output_paths=self.pklz_dirs,
            dry_run=False)

    def evaluation(self, special_evaluation_function=None):
        """
        :param special_evaluation_function: is pointer to function with
        fallowing signature:
            eval_dict = special_evaluation_function(volume1, volume2)
        """
        sliver_eval_all_to_yamls(
            self.yaml_files,
            self.pklz_dirs,
            self.sliver_dir,
            self.eval_files,
            recalculateThis=None,
            special_evaluation_function=special_evaluation_function
        )

    def report(self):
        self.dataframe_all = report(self.eval_files, self.labels, self.markers,
               show=self.show, output_prefix=self.image_basename,
               use_plt=self.use_plt, experiment_name=self.experiment_name,
               filename_separator=self.filename_separator
               )

    def is_evaluation_necessary(self):
        """
        Check if evaluation output file exists.
        """
        for filen in self.eval_files:
            if not os.path.isfile(filen + '.pkl'):
                return True
        return False

    def is_run_experiments_necessary(self):
        """
        Check if run_experiments output files exists. Number of files in
        every experiment dir is the same as number of files with seeds.
        """
        n_seeds = len(glob.glob(self.input_data_path_pattern))
        for pklz_dir in self.pklz_dirs:
            n_out_files = len(glob.glob(os.path.join(pklz_dir, '*.pkl*')))
            if n_out_files < n_seeds:
                return True
        return False

    def get_dataframe(self):
        """
        Dataframe is constructed in report() function
        :return: all experiments in pandas dataframe
        """
        return self.dataframe_all


def run_and_make_report(*pars, **params):
    rr = RunAndMakeReport(*pars, **params)
    rr.make_all()
    return rr.get_dataframe()


def generate_configs(pklz_dirs, conf_default, conf_list):
    """
    Based on conf_list and conf_default generate configs.

    """
    exp_conf_files = [os.path.normpath(path) + '.config' for path in pklz_dirs]
    # exp_conf_files = [os.path.join(os.path.normpath(path),
    for i in range(0, len(pklz_dirs)):
        conf = copy.copy(conf_default)
        conf.update(conf_list[i])
        io3d.misc.obj_to_file(conf, exp_conf_files[i], filetype='yaml')


def run_all_liver_segmentation_experiments_with_conf(
    exp_conf_files,
    input_data_path_pattern,
    output_paths,
    dry_run=False,
    force_run=False
):
    """
    Only if there is almost empty dir
    """

    for i in range(0, len(exp_conf_files)):

        config_file_path = exp_conf_files[i]
        # head, teil = os.path.split(config_file_path)
        # TODO make more robust
        # look into directory, if there are some file, we expect that this are
        # results

        if os.path.isfile(config_file_path):
            logger.debug('file "%s" found' % (config_file_path))
            n_in_files = len(glob.glob(input_data_path_pattern))
            n_out_files = len(glob.glob(output_paths[i] + '/*.pkl*'))
            if (n_out_files < n_in_files) or force_run:
                logger.debug('performing segmentation experiment')
                run_liver_segmentation_experiment_with_conf(
                    config_file_path,
                    input_data_path_pattern=input_data_path_pattern,
                    output_path=output_paths[i],
                    dry_run=dry_run)
            else:
                logger.info('skipping dir "%s"' % (output_paths[i]))

        else:

            confs = \
                'config file "' + config_file_path +\
                '" does not exist. Create it with "lisa -cf" parameter.'
            logger.warning(confs)

            print confs


def run_liver_segmentation_experiment_with_conf(
        config_file_path=None,
        bsh_lisa="python ./lisa.py -ni ",
        input_data_path_pattern="/home/mjirik/data/medical/processed/spring2014/exp010-seeds/org-liver-orig???.mhd-exp010-seeds.pklz",  # noqa
        output_path=None,
        dry_run=False,
        use_subprocess=False):
    """
    Run experiments with defined config file
    config_file_path:
    dry_run: only print command

    Example::
        run_experiment_with_conf(
            "/home/mjirik/exp026-03bl06sm/organ_segmentation.config",
            dry_run=True)
    """
#     bsh_lisa = "python /home/mjirik/projects/lisa/lisa.py -ni "
# bsh_data = "-dd
# /home/mjirik/data/medical/processed/spring2014/exp010-seeds/org-liver-orig%03i.mhd-exp010-seeds.pklz
# "

    if output_path is None:
        output_path, teil = os.path.split(config_file_path)

    if not os.path.isdir:
        os.mkdir(output_path)
    bsh_output = " -op " + output_path + " "
    filenames = glob.glob(input_data_path_pattern)
    if len(filenames) == 0:
        logger.warning('input_data_path_pattern "%s" is empty'
                       % (input_data_path_pattern))
        # print "input data path is empty " + input_data_path_pattern
    for fn in filenames:
        bsh_data = " -dd " + fn + " "

        bsh = bsh_lisa + bsh_data
        if config_file_path is not None:
            bsh_config = "-cf " + config_file_path
            bsh = bsh + bsh_config + bsh_output

        print bsh
        if not dry_run:
            if use_subprocess:
                process = subprocess.Popen(bsh.split(), stdout=subprocess.PIPE)
                output = process.communicate()[0]
                print output
            else:
                import lisa.organ_segmentation
                import sys
                try:
                    tmpargv = sys.argv
                    sys.argv = bsh.split()[1:]
                    lisa.organ_segmentation.main()
                    sys.argv = tmpargv
                except:
                    traceback.print_exc()

# <codecell>


def __create_data_frames(tables, indexes, columns):
    dfs = []
    for i in range(0, len(tables)):
        df = pandas.DataFrame(
            tables[i],
            index=indexes[i],
            columns=columns[i])
        dfs.append(df)
    return dfs


def __save_data_frames(dfs, labels, output_prefix):
    for df, label in zip(dfs, labels):
        fname = output_prefix + '-score-' + label.replace(' ', '') + '.tex'
        fo = open(fname, "w")
        fo.write(df.to_latex())
        fo.close()


def create_data_frame(data, labels, pklz_dirs, experiment_name=''):
    """
    :data: Create pandas dataframe

    data = [misc.obj_from_file(fn + '.pkl',filetype='pkl') for fn in eval_files]
    """

    scoreTotal, scoreMetrics, scoreAll = sliverScoreAll(data)

    df_pieces = []
    for (dat, label, pklz_dir, score_data) in \
            zip(data, labels, pklz_dirs, scoreAll):
        # add some columns
        dat['label'] = [label] * len(dat['avgd'])
        dat['pklz_dir'] = [pklz_dir] * len(dat['avgd'])
        dat['experiment_name'] = [experiment_name] * len(dat['avgd'])

        dat['avgd_pts'] = score_data['avgd']
        dat['maxd_pts'] = score_data['maxd']
        dat['voe_pts'] = score_data['voe']
        dat['vd_pts'] = score_data['vd']
        dat['rmsd_pts'] = score_data['rmsd']

        to_mean = [score_data['avgd'],
             score_data['maxd'],
             score_data['voe'],
             score_data['vd'],
             score_data['rmsd']]
        dat['score_pts'] = np.mean(to_mean, axis=0)
        df = pandas.DataFrame(dat, columns=dat.keys())
        df_pieces.append(df)

    df_all = pandas.concat(df_pieces)
    a, inv = np.unique(df_all['file1'], return_inverse=True)
    df_all['data_index'] = inv
    df_all.index = range(0, len(inv))
    return df_all


def __df_to_csv_and_latex(
    df_all, output_prefix, latex_columns=['vd_pts', 'voe_pts', 'avgd_pts',
                                          'maxd_pts', 'rmsd_pts',
                                          'experiment_name', 'label']
):
    with open(output_prefix + "all_data.csv", 'w') as f:
        df_all.to_csv(f, index=False)
    with open(output_prefix + "all_data.tex", 'w') as f:
        df_all[latex_columns].to_latex(f)


def report(eval_files, labels, markers, show=True, output_prefix='',
           use_plt=True, pklz_dirs=None, experiment_name='',
           filename_separator=''):
    """

    based on
    :eval_files: list of filenames with input data without extension. If it is
    None, based on pklz_disr the default eval_list is generated.
    :use_plt: can supress using of matplotlib
    :pklz_dirs: is used for generating eval_files if eval_files is None.
    Obsolete
    """

# TODO image_basename  generovat obrazky
    expn = np.array(range(0, len(markers)))
    expn_labels = labels
    dp_params = {
        'markers': markers,
        'labels': labels,
        'loc': 0,
        'show': show,
        'filename': output_prefix,
        'filename_separator': filename_separator
    }
    sp_params = {
        'expn': expn,
        'expn_labels': expn_labels,
        'show': show,
        'filename': output_prefix,
        'use_plt': use_plt,
        'filename_separator': filename_separator
    }
    # return
    # yaml_files = [os.path.normpath(path) + '.yaml' for path in pklz_dirs]
    # logger.debug(str(yaml_files))

    if eval_files is None:
        if pklz_dirs is not None:
            eval_files = [
                os.path.normpath(path) + '_eval' for path in pklz_dirs]
        else:
            logger.error('pklz_dirs and eval_files is None')
            raise Exception('pklz_dirs and eval_files is None')

    logger.debug(str(eval_files))
    data = [misc.obj_from_file(fname + '.pkl', filetype='pkl')
            for fname in eval_files]

    df_all = create_data_frame(data, labels, eval_files, experiment_name)
    __df_to_csv_and_latex(df_all, output_prefix)

    if use_plt:
        logger.debug("first plot")

        dataplot(data, 'voe', 'Volume Difference Error [%]', **dp_params)
        dataplot(data, 'vd', 'Total Volume Difference [%]', **dp_params)
        dataplot(data, 'processing_time', 'Processing time [s]', **dp_params)
        dataplot(data, 'maxd', 'MaxD [mm]', **dp_params)
        dataplot(data, 'avgd', 'AvgD [mm]', **dp_params)
        dataplot(data, 'rmsd', 'RMSD [mm]', **dp_params)
        # dataplot(data, 'rmsd', 'RMSD [mm]', **dp_params)

        logger.debug("Souhrn měření")

    vd_mn, tmp = sumplot(data, 'vd', 'Total Volume Difference', **sp_params)
    voe_mn, tmp = sumplot(data, 'voe', 'Volume Difference Error',
                          **sp_params)
    avgd_mn, tmp = sumplot(data, 'avgd', 'Average Distance', **sp_params)
    maxd_mn, tmp = sumplot(data, 'maxd', 'Maxiamal Distance', **sp_params)
    rmsd_mn, tmp = sumplot(data, 'rmsd', 'Square Distance', **sp_params)

    logger.info("\n")
    logger.info('vd   ' + str(vd_mn))
    logger.info("voe " + str(voe_mn))
    logger.info('maxd ' + str(maxd_mn))
    logger.info('avgd ' + str(avgd_mn))
    logger.info('rmsd ' + str(rmsd_mn))

    logger.info("Přepočteno na skóre")
    # print tables[0].shape
    # pandas.set_option('display.max_columns', None)
    scoreTotal, scoreMetrics, scoreAll = sliverScoreAll(data)

    tables, indexes, columns = scoreTableEvaluation(scoreMetrics)

    dataframes = __create_data_frames(tables, indexes, columns)
    __save_data_frames(dataframes, labels, output_prefix)

    # df = pandas.DataFrame(tables[0], index=indexes[0], columns=columns[0])
    # print df.to_string()

    if use_plt:
        dataplot(scoreAll, 'voe', 'Volume Difference Error [points]',
                 **dp_params)
        dataplot(scoreAll, 'vd', 'Total Volume Difference [points]',
                 **dp_params)

        dataplot(scoreAll, 'maxd', 'MaxD [points]', **dp_params)
        dataplot(scoreAll, 'avgd', 'AvgD [points]', **dp_params)
        dataplot(scoreAll, 'rmsd', 'RMSD [points]', **dp_params)
        # tohle je zoufale propletani ruznych dat
        dataplot(data, 'score_pts', 'Score [points]', **dp_params)

    vd_mn, tmp = sumplot(scoreAll, 'vd', 'Total Volume Difference', **sp_params)
    voe_mn, tmp = sumplot(scoreAll, 'voe', 'Volume Difference Error',
                          **sp_params)
    avgd_mn, tmp = sumplot(scoreAll, 'avgd', 'Average Distance', **sp_params)
    maxd_mn, tmp = sumplot(scoreAll, 'maxd', 'Maxiamal Distance', **sp_params)
    rmsd_mn, tmp = sumplot(scoreAll, 'rmsd', 'Square Distance', **sp_params)

    logger.info("Overall score")

    scoreTotal, scoreMetrics, scoreAll = sliverScoreAll(data)
    logger.info('Overall score: ' + str(scoreTotal))

    if use_plt:
        plot_total(scoreMetrics, labels=labels, err_scale=0.05, show=show,
                   filename=output_prefix)

    return df_all


def recalculate_suggestion(eval_files):
    """
    Check which evaluation does not exists
    """
    recalculateThis = []
    for i in range(0, len(eval_files)):
        if not os.path.isfile(eval_files[i]):
            recalculateThis.append(i)
    return recalculateThis


def sliver_eval_all_to_yamls(yaml_files, pklz_dirs, sliver_dir, eval_files,
                             recalculateThis=None,
                             special_evaluation_function=None):
    """
    This is time consuming.
    It can be specified which should be evaluated with recalculateThis=[2,5]

    :param yaml_files: is list of paths to output yaml files which will be created in this function
    :param special_evaluation_function: evaluation function like fallowing:
        eval_dict = special_evaluation_function(volume1, volume2, voxelsize_mm)
    """
    if recalculateThis is None:
        recalculateThis = recalculate_suggestion(eval_files)
    logger.debug('eval files ' + str(eval_files))

    for i in recalculateThis:
        logger.info("Performing evaluation on: " + str(pklz_dirs[i]))
        a, data = volumetry_evaluation.evaluate_and_write_to_file(
            yaml_files[i],
            pklz_dirs[i],
            sliver_dir,
            eval_files[i],
            visualization=False,
            return_dir_lists=True,
            special_evaluation_function=special_evaluation_function,
            return_all_data=True

        )
        logger.debug(str(a))
        # print data



def plotone(data, expn, keyword, ind, marker, legend):
    if ind in expn:
        xdata = range(1, len(data[ind][keyword]) + 1)
        plt.plot(
            xdata, data[ind][keyword], marker, label=legend)
        # , alpha=0.7, ms=10)


def dataplot(data, keyword, ylabel, expn=None, markers=None, labels=None,
             ymin=None, loc=0, filename='', show=True, filename_separator='-'):
    """
    Plot data. Function is prepared for our dataset (for example 5 measures).

    """
    print data[0].keys()
    if expn is None:
        expn = range(0, len(data))
    if markers is None:
        markers = ['kp'] * len(data)

    if labels is None:
        labels = [''] * len(data)

    for i in range(0, len(expn)):
        try:
            marker = markers[i]
            label = labels[i]
        except:
            marker = 'xr'
            label = 'Unknown label'
            pass
        plotone(data, expn, keyword, expn[i], marker, label)

    x1, x2, y1, y2 = plt.axis()
    # necheme grafy od 40 do 50. chceme grafy od nuly
    if ymin is not None:
        if y1 > 0:
            y1 = ymin
    plt.axis((0, x2 + 1, y1, y2))
    plt.ylabel(ylabel)
    plt.legend(numpoints=1, loc=loc,
               bbox_to_anchor=(1.05, 1), borderaxespad=0.)
    plt.savefig(filename + filename_separator + keyword + '.pdf',
                bbox_inches='tight')
    if show:
        plt.show()
    plt.close()


def sumplot(data, keyword, ylabel, expn=None, expn_labels=None, loc=70,
            filename='', labels_rotation=70, ymin=None, show=True,
            use_plt=True, filename_separator='-'):
    """
    Plot data. Function is prepared for our dataset (for example 5 measures).
    expn_labels: Labels for x-axis based aligned to expn
    """
    # print 'new sumplot'
    if expn is None:
        expn = range(0, len(data))

    # plotone(data, expn, keyword, 0, 'ks', '1 gauss')
    # plotone(data, expn, keyword, 1, 'kv', '3 gauss')
    # plotone(data, expn, keyword, 2, 'kp', 'smoothing')
    # plotone(data, expn, keyword, 3, 'k*', 'blowup')

    mn = []
    va = []

    for ind in expn:
        dat = data[ind][keyword]
        mn.append(np.mean(dat))
        va.append(np.var(dat))
    if use_plt:
        plt.errorbar(expn, mn, fmt='ks', yerr=va)
        x1, x2, y1, y2 = plt.axis()
        # necheme grafy od 40 do 50. chceme grafy od nuly

        if ymin is not None:
            if y1 > 0:
                y1 = ymin
        plt.axis((x1, x2, y1, y2))
        plt.ylabel(ylabel)
        plt.xlim([np.min(expn) - 1, np.max(expn) + 1])
        # plt.xticks([1,2,3],['hu','ha', 'te'])

        if expn_labels is not None:
            expn_labels = np.array(expn_labels)
            expn_labels = expn_labels[np.array(expn)]

            plt.xticks(expn, expn_labels, rotation=labels_rotation)
        # plt.legend(numpoints=1, loc=loc)
        # plt.savefig('pitomost.png')
        plt.savefig(filename + filename_separator + keyword + '.pdf',
                    bbox_inches='tight')
        # plt.savefig('-avgd.png')
        if show:
            plt.show()
        plt.close()

    return mn, va


def plot_total(scoreMetrics, err_scale=1, expn=None, labels=None,
               labels_rotation=70, ymin=None, filename=None, show=True):
    """
    err_scale: scale factor of error bars, defalut err_scale=1
    ymin: minimum of graph. If it is set to None, it is automatic
    """
    ylabel = 'Overal Score, err bars %.2f of variance' % (err_scale)
    if expn is None:
        expn = range(0, len(scoreMetrics))

    # print scoreMetrics

    mn = np.array([np.mean(oneset.reshape(-1)) for oneset in scoreMetrics])
    va = np.array([np.var(oneset.reshape(-1))
                   for oneset in scoreMetrics]) * err_scale

    plt.errorbar(expn, mn, fmt='ks', yerr=va)
    x1, x2, y1, y2 = plt.axis()
    # necheme grafy od 40 do 50. chceme grafy od nuly

    if ymin is not None:
        if y1 > 0:
            y1 = ymin
    plt.axis((x1, x2, y1, y2))
    plt.ylabel(ylabel)
    plt.xlim([np.min(expn) - 1, np.max(expn) + 1])
    # plt.xticks([1,2,3],['hu','ha', 'te'])

    if labels is not None:
        # expn_labels = np.array(['1 gauss', '3 gauss', 'smoothing', 'blowup'])
        labels = np.array(labels)
        labels = labels[np.array(expn)]

        plt.xticks(expn, labels, rotation=labels_rotation)

    if filename is not None:
        plt.savefig(filename + '-total.pdf', bbox_inches='tight')

    return mn, va


def sliverScore(measure, metric_type):
    """
    Based on sliver metodics
    http://sliver07.org/p7.pdf

    Slope and intercept comutations:
    https://docs.google.com/spreadsheet/ccc?key=0AkBzbxly5bqfdEJaOWJJUEh5ajVJM05YWGdaX1k5aFE#gid=0

    """
    slope = -1
    intercept = 100

    if metric_type is 'vd':
        slope = -3.90625
    elif metric_type is 'voe':
        slope = -5.31914893617021
    elif metric_type is 'avgd':
        slope = -25
    elif metric_type is 'rmsdd':
        slope = -14, 7058823529412
    elif metric_type is 'maxdd':
        slope = -1, 31578947368421

    score = intercept + np.abs(measure) * slope
    score[score < 0] = 0

    return score


def sliverScoreAll(data):  # , returnScoreEachData=False):
    """
    Computers score by Sliver07
    http://sliver07.org/p7.pdf

    input: dataset = [{'vd':[1.1, ..., 0.1], 'voe':[...], 'avgd':[...],
                      'rmsd':[...], 'maxd':[...]},
                      {'vd':[...], 'voe':[...], ...}
                     ]
    return: scoreTotal, scoreMetrics, scoreAll
        Order of scoreMetrics is [vd, voe, avgd, rmsd, maxd]


    """

    scoreAll = []
    scoreTotal = []
    scoreMetrics = []
    for dat in data:
        score = {
            'vd': sliverScore(dat['vd'], 'vd'),
            'voe': sliverScore(dat['voe'], 'voe'),
            'avgd': sliverScore(dat['avgd'], 'avgd'),
            'rmsd': sliverScore(dat['rmsd'], 'rmsd'),
            'maxd': sliverScore(dat['maxd'], 'maxd'),
        }
        scoreAll.append(score)

        metrics = np.array([
            score['vd'],
            score['voe'],
            score['avgd'],
            score['rmsd'],
            score['maxd']
        ])
        scoreMetrics.append(metrics)

        total = np.mean(np.mean(metrics))
        scoreTotal.append(total)

    # if returnScoreEachData:
    #    return scoreTotal, scoreMetrics, scoreAll, scoreEachData
    return scoreTotal, scoreMetrics, scoreAll


# print scoreMetrics
def scoreTableEvaluation(scoreMetrics):
    tables = []
    indexes = []
    columns = []
    for scoreMetric in scoreMetrics:
        score0 = np.mean(scoreMetric, 0)
        # print scoreMetric.shape
        score0 = score0.reshape([1, scoreMetric.shape[1]])
        # print score0.shape

        scm = np.vstack((scoreMetric, score0))
        #
        score1 = np.mean(scm, 1)
        score1 = score1.reshape([scm.shape[0], 1])
        # print 'sc1', score1
        scm = np.hstack((scm, score1))
        # print 'scm', scm
        tables.append(scm.T)

        # index and column
        index = range(1, scm.shape[1])
        index.append('score')
        # print 'len index', len(index)
        indexes.append(index)

        column = ['VD', 'VOE', 'AvgD', 'RMSD', 'MaxD', 'score']
        # range(1, scm.shape[0])
        # column.append('score')
        # print 'len col', len(column)
        columns.append(column)
        # print scm

    return tables, indexes, columns


def processIt(pklz_dirs, sliver_dir, yaml_files, eval_files, markers, labels):
    import misc
    """
    Funkce vypíše report o celém experimentu.
    """
    data = [misc.obj_from_file(fname + '.pkl', filetype='pkl')
            for fname in eval_files]

    print "Jednotlivá měření"
    dataplot(data, 'voe', 'Volume Difference Error [%]', markers=markers,
             labels=labels, loc=0)
    dataplot(data, 'vd', 'Total Volume Difference [%]', markers=markers,
             labels=labels, loc=0)

    dataplot(data, 'processing_time',
             'Processing time [s]', markers=markers, labels=labels, loc=0)

    dataplot(data, 'maxd', 'MaxD [mm]', markers=markers, labels=labels, loc=0)
    dataplot(data, 'avgd', 'AvgD [mm]', markers=markers, labels=labels, loc=0)
    dataplot(data, 'rmsd', 'RMSD [mm]', markers=markers, labels=labels, loc=0)

    print "Souhrn měření"

    # import "experiment_support.ipynb"

    expn = np.array(range(0, len(labels)))
    expn_labels = labels

    print expn_labels
    print expn

    vd_mn, tmp = sumplot(
        data, 'vd', 'Total Volume Difference', expn, expn_labels)
    voe_mn, tmp = sumplot(
        data, 'voe', 'Volume Difference Error', expn, expn_labels)

    avgd_mn, tmp = sumplot(data, 'avgd', 'Average Distance', expn, expn_labels)
    maxd_mn, tmp = sumplot(
        data, 'maxd', 'Maxiamal Distance', expn, expn_labels)
    rmsd_mn, tmp = sumplot(data, 'rmsd', 'Square Distance', expn, expn_labels)

    print 'vd   ', vd_mn
    print "voe ", voe_mn
    print 'maxd ', maxd_mn
    print 'avgd ', avgd_mn
    print 'rmsd ', rmsd_mn

    print "Přepočteno na skóre"

    import pandas
    # print tables[0].shape
    # pandas.set_option('display.max_columns', None)
    scoreTotal, scoreMetrics, scoreAll = sliverScoreAll(data)

    tables, indexes, columns = scoreTableEvaluation(scoreMetrics)

    df = pandas.DataFrame(tables[0], index=indexes[0], columns=columns[0])
    print df.to_string()
    dataplot(scoreAll, 'voe', 'Volume Difference Error [points]',
             markers=markers, labels=labels, loc=0)
    dataplot(scoreAll, 'vd', 'Total Volume Difference [points]',
             markers=markers, labels=labels, loc=0)

    dataplot(scoreAll, 'maxd',
             'MaxD [mm]', markers=markers, labels=labels, loc=0)
    dataplot(scoreAll, 'avgd',
             'AvgD [mm]', markers=markers, labels=labels, loc=0)
    dataplot(scoreAll, 'rmsd',
             'RMSD [mm]', markers=markers, labels=labels, loc=0)

    vd_mn, tmp = sumplot(
        scoreAll, 'vd', 'Total Volume Difference', expn, expn_labels)
    voe_mn, tmp = sumplot(
        scoreAll, 'voe', 'Volume Difference Error', expn, expn_labels)
    avgd_mn, tmp = sumplot(
        scoreAll, 'avgd', 'Average Distance', expn, expn_labels)
    maxd_mn, tmp = sumplot(
        scoreAll, 'maxd', 'Maxiamal Distance', expn, expn_labels)
    rmsd_mn, tmp = sumplot(
        scoreAll, 'rmsd', 'Square Distance', expn, expn_labels)

    # scoreTotal, scoreMetrics, scoreAll =
    # volumetry_evaluation.sliverScoreAll(data)
    scoreTotal, scoreMetrics, scoreAll = sliverScoreAll(data)
    print 'Score total: ', scoreTotal

    plot_total(scoreMetrics, labels=labels, err_scale=0.05)
    return df


def get_subdirs(dirpath, wildcard='*', outputfile='experiment_data.yaml'):

    dirlist = []
    if os.path.exists(dirpath):
        logger.info('dirpath = ' + dirpath)
        # print completedirpath
    else:
        logger.error('Wrong path: ' + dirpath)
        raise Exception('Wrong path : ' + dirpath)

    dirpath = os.path.abspath(dirpath)
    # print 'copmpletedirpath = ', completedirpath
    # import pdb; pdb.set_trace()
    dirlist = {
        o: {'abspath': os.path.abspath(os.path.join(dirpath, o))}
        for o in os.listdir(dirpath) if os.path.isdir(
            os.path.join(dirpath, o))
    }
    # import pdb; pdb.set_trace()

    # print [o for o in os.listdir(dirpath) if
    # os.path.isdir(os.path.abspath(o))]

    #    dirlist.append(infile)
    # print "current file is: " + infile
    misc.obj_to_file(dirlist, 'experiment_data.yaml', 'yaml')
    return dirlist

# Funkce vrati cast 3d dat. Funkce ma tri parametry :
# data - puvodni 3d data
# sp - vektor udavajici zacatek oblasti, kterou chceme nacist napr [10,20,2]
# Poradi : [z,y,x]
# area - (area size) udava velikost oblasti, kterou chceme vratit. Opet
# vektor stejne jako u sp
# Funkce kontroluje prekroceni velikosti obrazku.


def getArea(data, sp, area):
    if((sp[0] + area[0]) > data.shape[0]):
        sp[0] = data.shape[0] - area[0] - 1
        print "Funkce getArea() : Byla prekrocena velikost dat v ose Z"
    if((sp[1] + area[1]) > data.shape[1]):
        sp[1] = data.shape[1] - area[0] - 1
        print "Funkce getArea() : Byla prekrocena velikost dat v ose Y"
    if((sp[2] + area[2]) > data.shape[2]):
        sp[2] = data.shape[2] - area[0] - 1
        print "Funkce getArea() : Byla prekrocena velikost dat v ose X"
    return data[sp[0]:sp[0] + area[0],
                sp[1]:sp[1] + area[1],
                sp[2]:sp[2] + area[2]]


def setArea(data, sp, area, value):

    if((sp[0] + area[0]) > data.shape[0]):
        sp[0] = data.shape[0] - area[0] - 1
        print "Funkce getArea() : Byla prekrocena velikost dat v ose Z"
    if((sp[1] + area[1]) > data.shape[1]):
        sp[1] = data.shape[1] - area[0] - 1
        print "Funkce getArea() : Byla prekrocena velikost dat v ose Y"
    if((sp[2] + area[2]) > data.shape[2]):
        sp[2] = data.shape[2] - area[0] - 1
        print "Funkce getArea() : Byla prekrocena velikost dat v ose X"

    data[sp[0]:sp[0] + area[0], sp[1]:sp[1]
         + area[1], sp[2]:sp[2] + area[2]] = value
    return data

if __name__ == "__main__":
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(description='Experiment support')
    parser.add_argument('--get_subdirs', action='store_true',
                        default=None,
                        help='path to data dir')
    parser.add_argument('-o', '--output', default=None,
                        help='output file name')
    parser.add_argument('-i', '--input', default=None,
                        help='input')
    args = parser.parse_args()

    if args.get_subdirs:
        if args.output is None:
            args.output = 'experiment_data.yaml'
        get_subdirs(dirpath=args.input, outputfile=args.output)


#    SectorDisplay2__()



#! /usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/sed3/"))
#sys.path.append(os.path.join(path_to_script, "../extern/"))
#import featurevector
import argparse

import logging
logger = logging.getLogger(__name__)

import misc
import sed3

def main():

    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    data3d_a_path = None # os.path.join(path_to_script, '../../../data/medical/data_orig/sliver07/training-part1/liver-orig001.mhd')
    data3d_b_path = None #os.path.join(path_to_script, '../../../data/medical/data_orig/sliver07/training-part1/liver-seg001.mhd')
    parser = argparse.ArgumentParser(
            description='Information about pkl/pklz file')
    parser.add_argument('pklzFile',
            help='path to data' )
    args = parser.parse_args()

    data = misc.obj_from_file(args.pklzFile, 'pickle')
    print data

    try:
        pyed = sed3.sed3(data['data3d'], contour=data['segmentation'])
        pyed.show()
    except:
        try:

            pyed = sed3.sed3(data['data3d'])
            pyed.show()
        except:
            print "Problem with visualization"





if __name__ == "__main__":
    main()



#! /usr/bin/python
# -*- coding: utf-8 -*-



# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
# sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
#sys.path.append(os.path.join(path_to_script, "../extern/"))
#import featurevector

import logging
logger = logging.getLogger(__name__)


#import apdb
#  apdb.set_trace();\
#import scipy.io
import numpy as np
import scipy

# ----------------- my scripts --------
import sed3
try:
    import dcmreaddata as dcmr
except:
    from pysegbase import dcmreaddata as dcmr

# try:
#     from pysegbase import pycut
# except:
#     logger.warning("Deprecated of pyseg_base as submodule")
#     import pycut
import argparse
import qmisc
import io3d
import scipy.ndimage.filters as filters
import scipy.signal
import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate
from io3d import datareader
from scipy.ndimage.measurements import label
from scipy.ndimage import morphology

import misc





class SupportStructureSegmentation():
    def __init__(self,
            data3d = None,
            voxelsize_mm = None,
            autocrop = True,
            autocrop_margin_mm = [10,10,10],
            modality = 'CT',
            slab = {'none':0,'llung':4, 'bone':8,'rlung':9,'heart':6,'diaphragm':5},
            maximal_lung_diff = 0.4,
            rad_diaphragm=4,
            smer=None,
            working_voxelsize_mm = [2.0, 2.0, 2.0]
        ):

        """
        Segmentaton of support structures for liver segmentatio based on
        locati0on prior.
        """



        self.orig_voxelsize_mm = voxelsize_mm
        self.orig_shape = data3d.shape
        self.working_voxelsize_mm = working_voxelsize_mm
        self.crinfo = [[0,-1],[0,-1],[0,-1]]
        self.slab = slab
        self.maximal_lung_diff = maximal_lung_diff
        self.rad_diaphragm=rad_diaphragm
        self.smer=smer


        # data resize
        self.data3d = qmisc.resize_to_mm(data3d, voxelsize_mm, working_voxelsize_mm)
        self.segmentation = np.zeros(self.data3d.shape , dtype=np.int8)
        self.resized = True
        # self.data3d = qmisc.resize_to_mm(data3d, voxelsize_mm, working_voxelsize_mm)
        self.voxelsize_mm = self.working_voxelsize_mm

        self.autocrop = autocrop
        self.autocrop_margin_mm = np.array(autocrop_margin_mm)
        self.autocrop_margin = self.autocrop_margin_mm/self.voxelsize_mm

        #import pdb; pdb.set_trace()

    def resize_back_to_orig(self):
        """
        Resize segmentation to original shape
        """
        if self.resized:
            self.segmentation = qmisc.resize_to_shape(self.segmentation, 
                                                      self.orig_shape)
            self.resized = False


    def run(self):
        logger.debug('bones')
        self.bone_segmentation()
        logger.debug('spine')
        self.spine_segmentation()
        logger.debug('lungs')
        self.lungs_segmentation()
        logger.debug('heart')
        self.heart_segmentation()

        self.resize_back_to_orig()



    def import_data(self, data):
        self.data = data
        self.data3d = data['data3d']
        self.voxelsize_mm = data['voxelsize_mm']

    def import_dir(self, datadir):
        reader = dcmr.DicomReader(datadir)
        self.data3d = reader.get_3Ddata()
        self.metadata = reader.get_metaData()
        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(datadir)
        self.voxelsize_mm = np.array(self.metadata['voxelsize_mm'])


    def convolve_structure_heart( self , size=9 ):
        a = np.zeros(( size , size , size ))
        c = int (np.floor( size / 2 ))
        a[c,c,c]=1
        structure = filters.gaussian_filter( a , self.voxelsize_mm )
        structure[:c,:,:] *= -1
        structure[c,:,:] = 0
        return structure

    

    def bone_segmentation(self, bone_threshold = 200):
#ipdb.set_trace()
        return np.array(self.data3d > bone_threshold)#.astype(np.int8)*self.slab['bone']

    def convolve_structure_spine(self, velikost = [300 , 2, 2]):
        structure = np.ones((int(velikost[0]/self.voxelsize_mm[0]),int(velikost[1]/self.voxelsize_mm[1]),int(velikost[2]/self.voxelsize_mm[2])))  
        return structure

    

    def spine_segmentation(self, bone_threshold= 330):
        seg_prub = filters.gaussian_filter(self.data3d, 5.0/np.asarray(self.voxelsize_mm))
        structure = self.convolve_structure_spine()
        
        seg_prub = filters.convolve(seg_prub, structure)
	
	
#seg_prub = scipy.signal.fftconvolve(seg_prub, self.convolve_structure_spine())
        maximum = np.amax(seg_prub)
        seg_prub = np.array(seg_prub > 0.55*maximum)
#seg_prub = seg_prub * np.array(self.data3d>=bone_threshold)
#import sed3
#ed = sed3.sed3(seg_prub)
#ed.show()
        self.segmentation = seg_prub
        pass


    def __spl(self, x,y):
        return interpolate.bisplev(x,y, tck)

    def __above_diaphragm_calculation(self, seg_prub, internal_resize_shape=[20, 20, 20], data_degradation=4):
        import multipolyfit as mpf
        # print seg_prub.dtype
        # seg_prub_tmp = misc.resize_to_shape(seg_prub, internal_resize_shape)
        # print seg_prub_tmp.dtype
        seg_prub_tmp = seg_prub
        z, x, y = np.nonzero(seg_prub_tmp)
# data degradation
        x = x[::data_degradation]
        y = y[::data_degradation]
        z = z[::data_degradation]
        # import PyQt4.QtCore
        # PyQt4.QtCore.pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT

        s = np.array([x , y]).T
        h = np.array(z)
        model = mpf.multipolyfit(s, h, self.rad_diaphragm, model_out = True)
        # tck = interpolate.bisplrep(x,y,z, s=10)
        ran = seg_prub.shape
        x = np.arange( ran[1] )
        y = np.arange( ran[2] )
        x, y = np.meshgrid( x, y)
        # sh = [len(x), len(y)]
##x, y = np.meshgrid(x, y)
        # z = np.asarray(map(self.__spl(x,y), x.reshape(-1), y.reshape(-1))).reshape(sh)


        z = np.floor(np.asarray(map(model, x.reshape(-1), y.reshape(-1)))).astype(int)
        x = x.reshape(z.shape)
        y = y.reshape(z.shape)
        cc = np.zeros(ran)
        for a in range(z.shape[0]):
            if (z[a] < ran[0]) and (z[a] >= 0):

                #self.segmentation[z[a], x[a], y[a]] = self.slab['diaphragm']
                if self.smer==0:
                    cc[z[a]+1:, x[a], y[a]] = 1
                else:
                    cc[:z[a]-1, x[a], y[a]] = 1

                # self.segmentation[z[a], x[a], y[a]] = self.slab['diaphragm']
                for b in range(z[a]+1, ran[0]):
                    cc[b, x[a], y[a]] = 1

        # cc = misc.resize_to_shape(cc, seg_prub.shape)
        return cc

    def heart_segmentation(self, heart_threshold = 0, top_threshold = 200):
        a=self.convolve_structure_heart()
        seg_prub = np.array(self.segmentation == self.slab['rlung'])+np.array(self.segmentation == self.slab['llung'])


        logger.debug('pred konvoluci')
        seg_prub = filters.convolve( (seg_prub-0.5) , a )
        logger.debug('po konvoluci')


# import sed3
# ed = sed3.sed3(seg_prub)
# ed.show()
        if self.smer==0:
            seg_prub = np.array(seg_prub<=-0.3)
        else:
            seg_prub = np.array(seg_prub<=0.3)
        cc = self.__above_diaphragm_calculation(seg_prub)
#ipdb.set_trace()
        plice1=np.array(self.segmentation==self.slab['llung'])
        z, x, y = np.nonzero(plice1)

        x1 = [0,0,0,0]
        y1 = [0,0,0,0]
        z1 = [0,0,0,0]
        x1[0]=np.min(x)
        x1[1]=np.max(x)
        y1[0]=np.min(y)
        y1[1]=np.max(y)
        z1[0]=np.max(z)
        z1[1]=np.min(z)
        plice2=np.array(self.segmentation==self.slab['rlung'])
        z, x, y = np.nonzero(plice2)
        x1[2]=np.min(x)
        x1[3]=np.max(x)
        y1[2]=np.min(y)
        y1[3]=np.max(y)
        z1[2]=np.max(z)
        z1[3]=np.min(z)
        mp=np.zeros(self.segmentation.shape)
        xmin=np.min(x1)
        xmax=np.max(x1)
        ymin=np.min(y1)
        ymax=np.max(y1)
        zmin=np.min(z1)
        zmax=np.max(z1)
        if self.smer==0:
            mp[zmin:,  xmin:xmax ,ymin:ymax]=1
        else:
            mp[:zmax,  xmin:xmax ,ymin:ymax]=1

        bones = np.array(self.data3d >= top_threshold)
        aaa = np.array(self.data3d >= heart_threshold)
        aaa = aaa - bones
        logger.debug('pred binary opening')
        aaa=morphology.binary_opening(aaa , iterations=self.iteration()+2).astype(self.segmentation.dtype)
        aaa = morphology.binary_erosion(aaa, iterations=self.iteration())	
        aaa=cc * aaa * mp

        lab , num = label(aaa)
        counts= [0]*(num+1)
        for x in range(1, num+1):
            a = np.sum(np.array(lab == x))
            counts[x] = a
        index= np.argmax(counts)
        aaa = np.array(lab==index)
        logger.debug('pred dilataci')
        aaa = morphology.binary_dilation(aaa, iterations=self.iteration())
        #self.segmentation= aaa
        self.segmentation = self.segmentation + aaa.astype(np.int8)*self.slab['heart']


    def iteration(self, sirka = 5):
        prumer= np.mean(self.voxelsize_mm)
        a = int(sirka / prumer)
        return a

    def orientation(self):
        if(self.segmentation.shape[0]%2==0):
            split1, split2 = np.split(self.segmentation, 2, 0)
            if (np.sum(split1) > np.sum(split2)):
                self.smer=1

            else:
                self.smer=0


        else:
            split1, split2 = np.split(self.segmentation[1:,:,:], 2, 0)
            if(np.sum(split1)>np.sum(split2)):
                self.smer=1

            else:
                self.smer=0



    def volume_count(self, seg_prub):
	labeled_seg , num_seg = label(seg_prub)
	counts= [0]*(num_seg+1)
	for x in range(1, num_seg+1):
	    a = np.sum(np.array(labeled_seg == x))
	    counts[x] = a
	z, x, y = labeled_seg.shape
	index=labeled_seg[self.iteration()+5,self.iteration()+5,self.iteration()+5]
	counts[index]=0
	index=labeled_seg[self.iteration()+5,self.iteration()+5,y-self.iteration()-5]
	counts[index]=0
	index=labeled_seg[self.iteration()+5,x-self.iteration()-5,self.iteration()+5]
	counts[index]=0
	index=labeled_seg[self.iteration()+5,x-self.iteration()-5,y-self.iteration()-5]
	counts[index]=0
	return counts, labeled_seg





    def lungs_segmentation(self, lungs_threshold = -360):
	seg_prub = np.array(self.data3d <= lungs_threshold)
	seg_prub = morphology.binary_closing(seg_prub , iterations=self.iteration()).astype(self.segmentation.dtype)
	seg_prub = morphology.binary_opening(seg_prub , iterations = 5)
	counts , labeled_seg=self.volume_count(seg_prub)
	#self.segmentation = seg_prub
	#for x in np.nditer(labeled_seg, op_flags=['readwrite']):
	#    if x[...]!=0:890/
	#    	counts[x[...]]=counts[x[...]]+1
	#index=np.argmax(counts) #pozadí
	#counts[index]=0
	index=np.argmax(counts) #jedna nebo obě plíce
	velikost1=counts[index]
	counts[index]=0
	index2=np.argmax(counts)# druhá plíce nebo nečo jiného
	velikost2=counts[index2]
	if (1.0-self.maximal_lung_diff)<= float(velikost2)/velikost1:
	    print("plice separované")
	else:
	    print("plice neseparované")
	    pocet=0
	    seg_prub = np.array(self.data3d <= lungs_threshold)
	    seg_prub = morphology.binary_closing(seg_prub , iterations=self.iteration()).astype(self.segmentation.dtype)
	    seg_prub = morphology.binary_opening(seg_prub , iterations = 5)
		
	    while not (1.0 - self.maximal_lung_diff) <= float(velikost2)/velikost1:
		seg_prub = morphology.binary_erosion(seg_prub,iterations=1)
		counts, labeled_seg =self.volume_count(seg_prub)
	    	index=np.argmax(counts) #jedna nebo obě plíce
	    	velikost1=counts[index]
	    	counts[index]=0
	    	index2=np.argmax(counts)# druhá plíce nebo nečo jiného
	    	velikost2=counts[index2]
		pocet=pocet+1
	    seg_prub = morphology.binary_dilation(self.segmentation,iterations=pocet).astype(self.segmentation.dtype)
	#self.segmentation = self.segmentation + np.array(labeled_seg==index).astype(np.int8)*self.slab['lungs']
	#self.segmentation = self.segmentation + np.array(labeled_seg==index2).astype(np.int8)*self.slab['lungs']
	plice1 = np.array(labeled_seg==index)
	z,x,y = np.nonzero(plice1)
	m1 = np.max(y)
	if m1<(self.segmentation.shape[1]/2):
	    self.segmentation = self.segmentation + np.array(labeled_seg==index).astype(np.int8)*self.slab['llung']
	    self.segmentation = self.segmentation + np.array(labeled_seg==index2).astype(np.int8)*self.slab['rlung']
	else:
	    self.segmentation = self.segmentation + np.array(labeled_seg==index).astype(np.int8)*self.slab['rlung']
	    self.segmentation = self.segmentation + np.array(labeled_seg==index2).astype(np.int8)*self.slab['llung']
	self.orientation()
	if self.smer==1:
	    self.segmentation[self.segmentation==self.slab['llung']]=3
	    self.segmentation[self.segmentation==self.slab['rlung']]=self.slab['llung']
	    self.segmentation[self.segmentation==3]=self.slab['rlung']
        pass



    def _crop(self, data, crinfo):
        """
        Crop data with crinfo
        """
        data = data[crinfo[0][0]:crinfo[0][1], crinfo[1][0]:crinfo[1][1], crinfo[2][0]:crinfo[2][1]]
        return data


    def _crinfo_from_specific_data (self, data, margin):
# hledáme automatický ořez, nonzero dá indexy
        nzi = np.nonzero(data)

        x1 = np.min(nzi[0]) - margin[0]
        x2 = np.max(nzi[0]) + margin[0] + 1
        y1 = np.min(nzi[1]) - margin[0]
        y2 = np.max(nzi[1]) + margin[0] + 1
        z1 = np.min(nzi[2]) - margin[0]
        z2 = np.max(nzi[2]) + margin[0] + 1

# ošetření mezí polí
        if x1 < 0:
            x1 = 0
        if y1 < 0:
            y1 = 0
        if z1 < 0:
            z1 = 0

        if x2 > data.shape[0]:
            x2 = data.shape[0]-1
        if y2 > data.shape[1]:
            y2 = data.shape[1]-1
        if z2 > data.shape[2]:
            z2 = data.shape[2]-1

# ořez
        crinfo = [[x1, x2],[y1,y2],[z1,z2]]
        #dataout = self._crop(data,crinfo)
        #dataout = data[x1:x2, y1:y2, z1:z2]
        return crinfo


    def im_crop(self, im,  roi_start, roi_stop):
        im_out = im[ \
                roi_start[0]:roi_stop[0],\
                roi_start[1]:roi_stop[1],\
                roi_start[2]:roi_stop[2],\
                ]
        return  im_out

    def export(self):
        slab={}
        slab['none'] = 0
	slab['heart']=10
        slab['llung'] = 4
	slab['rlungs'] = 9
        data = {}
        data['version'] = (1,0,0)
        data['data3d'] = self.data3d
        data['crinfo'] = self.crinfo
        data['segmentation'] = self.segmentation
        data['slab'] = slab
        data['voxelsize_mm'] = self.voxelsize_mm
        #import pdb; pdb.set_trace()
        return data




    def visualization(self):
        """
        Run viewer with output data3d and segmentation
        """

        try:
            from pysegbase.seed_editor_qt import QTSeedEditor
        except:
            logger.warning("Deprecated of pyseg_base as submodule")
            from seed_editor_qt import QTSeedEditor
        from PyQt4.QtGui import QApplication
        import numpy as np
#, QMainWindow
        app = QApplication(sys.argv)
        #pyed = QTSeedEditor(self.data3d, contours=(self.segmentation>0))
        pyed = QTSeedEditor(self.segmentation)
        pyed.exec_()


        #import pdb; pdb.set_trace()


        #pyed = QTSeedEditor(deletemask, mode='draw')
        #pyed.exec_()

        app.exit()






def main():

    #logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(description=
            'Segmentation of bones, lungs and heart.')
    parser.add_argument('-i','--datadir',
            default=None,
            help='path to data dir')
    parser.add_argument('-o','--output',
            default=None,
            help='output file')

    parser.add_argument('-d', '--debug', action='store_true',
            help='run in debug mode')
    parser.add_argument('-ss', '--segspine', action='store_true',
            help='run spine segmentaiton')
    parser.add_argument('-sl', '--seglungs', action='store_true',
            help='run lungs segmentation')
    parser.add_argument('-sh', '--segheart', action='store_true',
            help='run heart segmentation')
    parser.add_argument('-sb', '--segbones', action='store_true',
            help='run bones segmentation')
    parser.add_argument('-exd', '--exampledata', action='store_true',
            help='run unittest')
    parser.add_argument('-so', '--show_output', action='store_true',
            help='Show output data in viewer')
    args = parser.parse_args()



    if args.debug:
        logger.setLevel(logging.DEBUG)


    if args.exampledata:

        args.dcmdir = '../sample_data/liver-orig001.raw'

#    if dcmdir == None:

    #else:
    #dcm_read_from_dir('/home/mjirik/data/medical/data_orig/46328096/')
    #data3d, metadata = dcmr.dcm_read_from_dir(args.dcmdir)

    data3d , metadata = io3d.datareader.read(args.datadir)

    sseg = SupportStructureSegmentation(data3d = data3d,
            voxelsize_mm = metadata['voxelsize_mm'],
            )


    #sseg.orientation()
    if args.segbones:
        sseg.bone_segmentation()
    if args.segspine:
        sseg.spine_segmentation()
    if args.seglungs or args.segheart:
    	sseg.lungs_segmentation()
    if args.segheart:
    	sseg.heart_segmentation()


    sseg.resize_back_to_orig()
    #print ("Data size: " + str(data3d.nbytes) + ', shape: ' + str(data3d.shape) )

    #igc = pycut.ImageGraphCut(data3d, zoom = 0.5)
    #igc.interactivity()


    #igc.make_gc()
    #igc.show_segmentation()

    # volume
    #volume_mm3 = np.sum(oseg.segmentation > 0) * np.prod(oseg.voxelsize_mm)


    #pyed = sed3.sed3(oseg.data3d, contour = oseg.segmentation)
    #pyed.show()

    if args.show_output:
        sseg.visualization()

    #savestring = raw_input ('Save output data? (y/n): ')
    #sn = int(snstring)
    if args.output is not None: # savestring in ['Y','y']:
        import misc

        data = sseg.export()

        misc.obj_to_file(data, args.output, filetype = 'pickle')
    #output = segmentation.vesselSegmentation(oseg.data3d, oseg.orig_segmentation)


if __name__ == "__main__":
    main()



#! /usr/bin/python
# -*- coding: utf-8 -*-



# import funkcí z jiného adresáře
import sys
import os.path
path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
#import featurevector
import unittest

import logging
logger = logging.getLogger(__name__)

import numpy as np
import scipy.ndimage

# ----------------- my scripts --------
import misc
import sed3
import virtual_resection


def resection(data):
    #pyed = sed3.sed3(data['segmentation'])
    #pyed.show()
    # vessels = get_biggest_object(data['segmentation'] == data['slab']['porta'])
    vessels = data['segmentation'] == data['slab']['porta']
# ostranění porty z více kusů, nastaví se jim hodnota liver
    #data['segmentation'][data['segmentation'] == data['slab']['porta']] = data['slab']['liver']
    #show3.show3(data['segmentation'])
    import pdb; pdb.set_trace()

    #data['segmentation'][vessels == 1] = data['slab']['porta']
    #segm = data['segmentation']
    #pyed = sed3.sed3(vessels)
    #pyed.show()
    print ("Select cut")
    lab = virtual_resection.cut_editor_old(data)
    pyed = sed3.sed3(lab )#, contour=segm)
    pyed.show()
    l1 = 1
    l2 = 2
    #import pdb; pdb.set_trace()

    # dist se tady počítá od nul jenom v jedničkách
    dist1 = scipy.ndimage.distance_transform_edt(lab != l1)
    dist2 = scipy.ndimage.distance_transform_edt(lab != l2)




    #segm = (dist1 < dist2) * (data['segmentation'] != data['slab']['none'])
    segm = (((data['segmentation'] != 0) * (dist1 < dist2)).astype('int8') + (data['segmentation'] != 0).astype('int8'))

# vizualizace 1
#    pyed = sed3.sed3(segm)
#    pyed.show()
#    import pdb; pdb.set_trace()
#    pyed = sed3.sed3(data['data3d'], contour=segm)
#    pyed.show()
#    import pdb; pdb.set_trace()

# vizualizace 2
    linie = np.abs(dist1 - dist2) < 1
    pyed = sed3.sed3(data['data3d'], contour = data['segmentation']==data['slab']['liver'] ,seeds = linie)
    pyed.show()

    #show3.show3(data['segmentation'])








def get_biggest_object(data):
    """ Return biggest object """
    lab, num = scipy.ndimage.label(data)
    #print ("bum = "+str(num))

    maxlab = max_area_index(lab, num)

    data = (lab == maxlab)
    return data


def max_area_index(labels, num):
    """
    Return index of maxmum labeled area
    """
    mx = 0
    mxi = -1
    for l in range(1,num):
        mxtmp = np.sum(labels == l)
        if mxtmp > mx:
            mx = mxtmp
            mxi = l

    return mxi


import gtk
# import numpy as np
from matplotlib.patches import Polygon  # , PathPatch
from mpl_toolkits.mplot3d import art3d
from matplotlib.figure import Figure
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas

class SectorDisplay2__:
    def __init__(self):
        win = gtk.Window()
        win.set_default_size(800,800)
        vbox = gtk.VBox()
        win.add(vbox)

        fig = Figure()
        canvas = FigureCanvas(fig)  # a gtk.DrawingArea
        ax = fig.add_subplot(111, projection='3d')

        a = np.array([[0,0],[10,0],[10,10],[0,10]])
        p = Polygon(a,fill=True)
        ax.add_patch(p)
        art3d.pathpatch_2d_to_3d(p, z=3)

        ax.set_xlim3d(0, 20)
        ax.set_ylim3d(0, 20)
        ax.set_zlim3d(0, 20)

        vbox.pack_start(canvas)
        win.show_all()

# Run the Gtk mainloop
        gtk.main()

if __name__ == "__main__":
    data = misc.obj_from_file("vessels.pickle", filetype = 'pickle')
    #ds = data['segmentation'] == data['slab']['liver']
    #pyed = sed3.sed3(data['segmentation'])
    #pyed.show()
    resection(data)

#    SectorDisplay2__()




#! /usr/bin/python
# -*- coding: utf-8 -*-

"""
Surface model generator of vessel tree.
Used by gen_volume_tree.py
"""
import logging
logger = logging.getLogger(__name__)


# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../../lar-cc/lib/py/"))
import numpy as np

from larcc import VIEW, MKPOL, AA, INTERVALS
from splines import all
# import mapper
# from largrid import *

import geometry3d as g3
# import warnings
# warnings.filterwarnings('error')

class GTLar:
    """
    gtree is information about input data structure.
    endDistMultiplicator: move connected side of cylinders away from joint by multiplication of radius
    """
    def __init__(self, gtree=None,
                 endDistMultiplicator=0.5,
                 use_joints=True
                 ):
        
        logger.debug('__init__:use_joints = '+str(use_joints))
        logger.debug('__init__:endDistMultiplicator = '+str(endDistMultiplicator))
        
        # input of geometry and topology
        self.V = []
        self.CV = []
        self.joints = {}
        self.joints_lar = []
        self.gtree = gtree
        self.endDistMultiplicator = endDistMultiplicator
        self.use_joints = use_joints

    def add_cylinder(self, nodeA, nodeB, radius, cylinder_id=None):
        """
        cylinder_id is not needed anymore
        """        
        
        try:
            idA = tuple(nodeA)  # self.gtree.tree_data[cylinder_id]['nodeIdA']
            idB = tuple(nodeB)  # self.gtree.tree_data[cylinder_id]['nodeIdB']
        except:
            idA = 0
            idB = 0
            self.use_joints = False

        if all(nodeA == nodeB):
            logger.error("End points are on same place")
        
        # generate lists of points of two end circles
        ptsA, ptsB = g3.cylinder_circles(nodeA, nodeB, radius,
                                         element_number=30)
        
        # gives points unique global id. merges to global list of points.
        # self.joints[id] == list of lists of ids of cylinder points that 
        # belong to joint idA/idB.
        CVlistA = self.__construct_cylinder_end(ptsA, idA)
        CVlistB = self.__construct_cylinder_end(ptsB, idB)
        CVlist = CVlistA + CVlistB

        self.CV.append(CVlist)

    def __construct_cylinder_end(self, pts, id):
        """
        creates end of cylinder and prepares for joints
        """
        CVlist = []
        # base
        ln = len(self.V)

        for i, pt in enumerate(pts):
            self.V.append(pt)
            CVlist.append(ln + i)

        try:
            self.joints[id].append(CVlist)
        except:
            self.joints[id] = [CVlist]

        return CVlist

    def finish(self):
        """
        Generate joints for cylindr connections
        """
        if self.use_joints:
            logger.debug('generating joints...')
            
            for i, joint in enumerate(self.joints.values()):
                if i%10 == 0:
                    logger.debug('joint '+str(i)+'/'+str(len(self.joints.values())))
                # There is more then just one circle in this joint, so it
                # is not end of vessel
                if len(joint) > 1:
                    self.__generate_joint(joint)
                    
            logger.debug('joints generated')
                    
    def __get_cylinder_info_from_raw_joint(self, joint):
        """
        joint - joint value (list of lists), before being processed by finish(), or __generate_joint()
        
        Returns list of dictionaries with information about cylinders connected to current joint
        
            | cylinders[i]['near_points'] = list of point ids of circle on the side connected to current joint (node) 
            | cylinders[i]['far_points'] = list of point ids of circle on the other side of cylinder 
            | cylinders[i]['radius'] = radius of cylinder
            | cylinders[i]['near_node'] = position of node that is connected to current joint 
            | cylinders[i]['far_node'] = position of node on the other side of cylinder 
            | cylinders[i]['vector'] = vector of line connection between nodes
        """
        # get points of other side of cylinder, that is not connected to 
        # current joint
        cylinders = []
        for i, j in enumerate(joint):
            for cv_list in self.CV:
                if j[0] in cv_list:
                    far_points = []
                    for p in cv_list:
                        if p not in j:
                            far_points.append(p)
                                
                    cylinders.append({'near_points':j, 'far_points':far_points})               
                    break
        
        # get radiuses, node positions and vectors of connected cylinders
        for c in cylinders:
            p1 = np.array(self.V[c['near_points'][0]])
            p2 = np.array(self.V[c['near_points'][len(c['near_points'])/2]])
            
            dist = np.linalg.norm(p1-p2) / 2.0
            c['radius'] = dist
            
            # get position of nodes
            c['near_node'] = (p2 + ((p1-p2) / 2.0)).tolist()
            
            p1 = np.array(self.V[c['far_points'][0]])
            p2 = np.array(self.V[c['far_points'][len(c['far_points'])/2]])
            c['far_node'] = (p2 + ((p1-p2) / 2.0)).tolist()
            
            c['vector'] = (np.array(c['near_node']) - 
                                np.array(c['far_node'])).tolist()
                                
        return cylinders
        
    def __point_in_cylinder(self, c_nodeA, c_nodeB, radius, point, length_sq=None, radius_sq=None):
        """
        Tests if point is inside a cylinder
        http://www.flipcode.com/archives/Fast_Point-In-Cylinder_Test.shtml
        """
        if length_sq is None:
            if c_nodeA == c_nodeB:
                # wierd cylinder with 0 length
                logger.warning('__point_in_cylinder: distance between nodeA and nodeB is zero!!')
                raise Exception('distance between nodeA and nodeB is zero')
            length_sq = np.linalg.norm(np.array(c_nodeA)-np.array(c_nodeB))**2
        if radius_sq is None:
            radius_sq = radius**2
            
        dx = c_nodeB[0] - c_nodeA[0]	
        dy = c_nodeB[1] - c_nodeA[1]   
        dz = c_nodeB[2] - c_nodeA[2]

        pdx = point[0] - c_nodeA[0]	
        pdy = point[1] - c_nodeA[1]
        pdz = point[2] - c_nodeA[2]
        
        dot = pdx * dx + pdy * dy + pdz * dz
        
        if (dot < 0) or (dot > length_sq):
            # points is outside of end caps
            return False
        else:
            dsq = (pdx*pdx + pdy*pdy + pdz*pdz) - (dot*dot)/float(length_sq)
            
            if dsq > radius_sq:
                # point not inside cylinder
                return False
            else:
                return True
            

    def __generate_joint(self, joint):
        # get cylinder info
        cylinders = self.__get_cylinder_info_from_raw_joint(joint)
        
        # move connected side of cylinders away from joint by 
        # radius*self.endDistMultiplicator to create more place for joint
        for c in cylinders:
            if c['far_node'] == c['near_node']:
                # wierd cylinder with 0 length
                continue
            
            start_id = c['near_points'][0]
            end_id = c['near_points'][len(c['near_points'])-1]
            
            for p_id in range(start_id, end_id+1):
                self.V[p_id] = g3.translate(self.V[p_id], c['vector'],
                                    -c['radius']*self.endDistMultiplicator)
                # TODO - detect when g3.translate would create negative length
        # update cylinder info after moving points
        cylinders = self.__get_cylinder_info_from_raw_joint(joint)
                                    
        
        # cut out overlapping parts of cylinders (only in joint)
        new_V = list(self.V)
        for c in cylinders:
            # for every cylinder in joint...
            v = np.array(c['vector'])
            c_len = np.linalg.norm(np.array(c['near_node'])-np.array(c['far_node']))
            if c_len == 0:
                # wierd cylinder with 0 length
                continue
            
            for p_id in c['near_points']:
                # for every point in cylinder that is on the side connected to 
                # joint...
                orig_near_point = np.array(self.V[p_id])
                
                # get position of coresponding point on the far side for the 
                # point on the near side.
                # cylinder must be uncut for this to work correctly
                far_point = orig_near_point - v
                
                for cc in cylinders:
                    # for other cylinders connected to joint...
                    
                    if cc['near_points'] == c['near_points']:
                        # skip cylinder that owns tested point
                        continue 
                    elif cc['far_node'] == cc['near_node']:
                        # different cylinder, but has 0 length
                        continue
                    
                    current_point = far_point.copy()
                    current_point_last = far_point.copy()
                    
                    if self.__point_in_cylinder(cc['near_node'], cc['far_node'], cc['radius'], current_point):
                        continue # far point is inside !!! 
                    
                    while np.linalg.norm(current_point-far_point) <= c_len:
                        # slowly go from position of far_point to near_point, 
                        # and when the next step would be inside of cylinder, 
                        # set position of near_node to current position...
                        current_point_last = current_point.copy()
                        current_point = current_point + v/10.0 # move by 10% of length
                        
                        if self.__point_in_cylinder(cc['near_node'], cc['far_node'], cc['radius'], current_point):
                            if np.linalg.norm(current_point_last-far_point) < np.linalg.norm(np.array(new_V[p_id])-far_point):
                                new_V[p_id] = list(current_point_last)
                            break
                            
        self.V = list(new_V)
        
        
        # Takes all lists of points of circles that belong to joint and 
        # merge-copy them to one new list.
        # Points in list are covered with surface => this creates joint.
        joint = (np.array(joint).reshape(-1)).tolist()
        self.CV.append(joint)
        

    def show(self):
        V = self.V
        CV = self.CV

        VIEW(MKPOL([V, AA(AA(lambda k:k + 1))(CV), []]))

    def get_output(self):
        pass

    #def __add_tetr(self, nodeB):
        #"""
        #Creates tetrahedron in specified position.
        #"""
        #try:
            #nodeB = nodeB.tolist()
        #except:
            #pass

        #ln = len(self.V)
        #self.V.append(nodeB)
        #self.V.append((np.array(nodeB) + [2, 0, 0]).tolist())
        #self.V.append((np.array(nodeB) + [2, 2, 0]).tolist())
        #self.V.append((np.array(nodeB) + [2, 2, 2]).tolist())
        #self.CV.append([ln, ln + 1, ln + 2, ln + 3])

    #def __add_cone(self, nodeA, nodeB, radius):
        #vect = (np.array(nodeA) - np.array(nodeB)).tolist()
        #pts = self.__circle(nodeA, vect, radius)

        #ln = len(self.V)
        #self.V.append(nodeB)
        ## first object is top of cone
        #CVlist = [ln]

        #for i, pt in enumerate(pts):
            #self.V.append(pt)
            #CVlist.append(ln + i + 1)

        #self.CV.append(CVlist)

    #def __add_circle(self, center, perp_vect, radius, polygon_element_number=10):
        #"""
        #Draw circle some circle points as tetrahedrons.
        #"""
        #pts = g3.circle(center, perp_vect, radius,
                        #polygon_element_number=polygon_element_number)
        #for pt in pts:
            #self.__add_tetr(pt)



# /usr/bin/env python
# -*- coding: utf-8 -*-
"""

LISA - organ segmentation tool.

Liver Surgery Analyser

python organ_segmentation.py

python organ_segmentation.py -mroi -vs 0.6

Author: Miroslav Jirik
Email: miroslav.jirik@gmail.com


"""

import logging
logger = logging.getLogger(__name__)

import sys
import os
import os.path as op
# from collections import namedtuple
import exceptionProcessing

# from scipy.io import loadmat, savemat
import scipy
import scipy.ndimage
import numpy as np
import datetime
import argparse
import copy
# tady uz je logger
# import dcmreaddata as dcmreader
# from pysegbase import pycut
# try:
#     import pysegbase  # noqa
#     from pysegbase import pycut
# except:
#     path_to_script = os.path.dirname(os.path.abspath(__file__))
#     sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
#     logger.warning("Deprecated of pyseg_base as submodule")
#     import traceback
#     traceback.print_exc()
#     import pycut

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../../pysegbase/"))
import volumetry_evaluation

# from seg2fem import gen_mesh_from_voxels, gen_mesh_from_voxels_mc
# from viewer import QVTKViewer
import qmisc
import misc
import config
from io3d import datareader
from io3d import datawriter
import data_plus
import support_structure_segmentation as sss
import cachefile as cachef
import config_default
import liver_seeds

# import audiosupport
# import skimage
# import skimage.transform

scaling_modes = {
    'original': (None, None, None),
    'double': (None, 'x2', 'x2'),
    '3mm': (None, '3', '3')
}

#  Defaultparameters for segmentation

# version comparison
from pkg_resources import parse_version
import sklearn
if parse_version(sklearn.__version__) > parse_version('0.10'):
    # new versions
    cvtype_name = 'covariance_type'
else:
    cvtype_name = 'cvtype'

default_segmodelparams = {
    'type': 'gmmsame',
    'params': {cvtype_name: 'full', 'n_components': 3}
}

config_version = [1, 0, 0]


def import_gui():
    # from lisaWindow import OrganSegmentationWindow
    # from PyQt4.QtGui import QApplication, QMainWindow, QWidget,\
    #     QGridLayout, QLabel, QPushButton, QFrame, \
    #     QFont, QPixmap
    # from PyQt4.Qt import QString

    pass

def printTotals(transferred, toBeTransferred):
    print "Transferred: {0}\tOut of: {1}".format(transferred, toBeTransferred)

class OrganSegmentation():
    """
    Main object of Lisa user interface.
    """
    def set_params(self, *args, **kwargs):
        """
        Function set parameters in same way as constructor does

        :param args:
        :param kwargs:
        :return:
        """
        self.__init__(*args, **kwargs)

    def __init__(
        self,
        datapath=None,
        working_voxelsize_mm=3,
        viewermax=None,
        viewermin=None,
        series_number=None,
        autocrop=True,
        autocrop_margin_mm=[10, 10, 10],
        manualroi=False,
        texture_analysis=None,
        segmentation_smoothing=False,
        smoothing_mm=4,
        volume_blowup=1.00,
        data3d=None,
        metadata=None,
        seeds=None,
        edit_data=False,
        segparams={},
        segmodelparams=default_segmodelparams,
        roi=None,
        output_label=1,
        slab={},
        output_datapath=None,
        input_datapath_start='',
        experiment_caption='',
        lisa_operator_identifier='',
        volume_unit='ml',
        save_filetype='pklz',
        debug_mode=False,
        seg_postproc_pars={},
        cache_filename='cache.yml',
        seg_preproc_pars={},
        after_load_processing={},
        segmentation_alternative_params={},
        sftp_username='lisa_default',
        sftp_password=''


        #           iparams=None,
    ):
        """ Segmentation of objects from CT data.

        :param datapath: path to directory with dicom files
        :param manualroi: manual set of ROI before data processing, there is a
             problem with correct coordinates
        :param data3d, metadata: it can be used for data loading not from
        directory. If both are setted, datapath is ignored
        :param output_label: label for output segmented volume
        :param slab: aditional label system for description segmented data
        {'none':0, 'liver':1, 'lesions':6}
        :param roi: region of interest.
        [[startx, stopx], [sty, spy], [stz, spz]]
        :param seeds: ndimage array with size same as data3d
        :param experiment_caption = this caption is used for naming of outputs
        :param lisa_operator_identifier: used for logging
        :param input_datapath_start: Path where user directory selection dialog
            starts.
        :param volume_blowup: Blow up volume is computed in smoothing so it is
        working only if smoothing is turned on.
        :param seg_postproc_pars: Can be used for setting postprocessing
        parameters. For example
        :param segmentation_alternative_params: dict of alternative params f,e.
        {'vs5: {'voxelsize_mm':[5,5,5]}, 'vs3: {'voxelsize_mm':[3,3,3]}}
        """

        from pysegbase import pycut
        default_segparams = {
            'method': pycut.methods[0],
            'pairwise_alpha_per_mm2': 40,
            'use_boundary_penalties': False,
            'boundary_penalties_sigma': 50}

        self.iparams = {}
        self.datapath = datapath
        self.output_datapath = output_datapath
        self.sftp_username=sftp_username
        self.sftp_password=sftp_password
        self.input_datapath_start = input_datapath_start
        self.crinfo = [[0, None], [0, None], [0, None]]
        self.slab = data_plus.default_slab()
        self.slab.update(slab)
        self.output_label = output_label
        self.working_voxelsize_mm = None
        self.input_wvx_size = working_voxelsize_mm

        # print segparams
# @TODO each axis independent alpha
        self.segparams = default_segparams
        self.segparams.update(segparams)
        self.segmodelparams = default_segmodelparams
        self.segmodelparams.update(segmodelparams)
        self.series_number = series_number

        self.autocrop = autocrop
        self.autocrop_margin_mm = np.array(autocrop_margin_mm)
        self.texture_analysis = texture_analysis
        self.segmentation_smoothing = segmentation_smoothing
        self.smoothing_mm = smoothing_mm
        self.volume_blowup = volume_blowup
        self.edit_data = edit_data
        self.roi = roi
        self.data3d = data3d
        self.seeds = seeds
        self.segmentation = None
        self.processing_time = None
        self.experiment_caption = experiment_caption
        self.lisa_operator_identifier = lisa_operator_identifier
        self.version = qmisc.getVersionString()
        if self.version is None:
            self.version = "1.8.0"
        self.viewermax = viewermax
        self.viewermin = viewermin
        self.volume_unit = volume_unit
        self.organ_interactivity_counter = 0
        self.dcmfilelist = None
        self.save_filetype = save_filetype
        self.vessel_tree = {}
        self.debug_mode = debug_mode
        self.segmentation_alternative_params = segmentation_alternative_params
        # SegPostprocPars = namedtuple(
        #     'SegPostprocPars', [
        #         'smoothing_mm',
        #         'segmentation_smoothing',
        #         'volume_blowup',
        #         'snakes',
        #         'snakes_method',
        #         'snakes_params']
        # )
        self.cache = cachef.CacheFile(cache_filename)

        self.seg_postproc_pars = {
            'smoothing_mm': smoothing_mm,
            'segmentation_smoothing': segmentation_smoothing,
            'volume_blowup': volume_blowup,
            'snakes': False,
            'snakes_method': 'ACWE',
            'snakes_params': {'smoothing': 1, 'lambda1': 100, 'lambda2': 1},
            'snakes_niter': 20,
            # 'postproc_working_voxelsize': [1.0, 1.0, 1.0],
            'postproc_working_voxelsize': 'orig',
        }
        self.seg_postproc_pars.update(seg_postproc_pars)
        self.seg_preproc_pars = {
            'use_automatic_segmentation': True,
        }
        self.seg_preproc_pars.update(seg_preproc_pars)
        self.after_load_processing={
            'run_automatic_liver_seeds': False,
        }
        self.after_load_processing.update(after_load_processing)
        self.apriori = None
        # seg_postproc_pars.update(seg_postproc_pars)
        # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT

        # self.seg_postproc_pars = SegPostprocPars(**seg_postproc_pars_default)

#
        oseg_input_params = locals()
        oseg_input_params = self.__clean_oseg_input_params(oseg_input_params)

        logger.debug("oseg_input_params")
        logger.debug(str(oseg_input_params))
        self.oseg_input_params = oseg_input_params

        if data3d is None or metadata is None:
            # if 'datapath' in self.iparams:
            #     datapath = self.iparams['datapath']

            if datapath is not None:
                reader = datareader.DataReader()
                datap = reader.Get3DData(datapath, dataplus_format=True)
                # self.iparams['series_number'] = metadata['series_number']
                # self.iparams['datapath'] = datapath
                self.import_dataplus(datap)

        else:
            # self.data3d = data3d
            # default values are updated in next line
            mindatap = {'series_number': -1,
                        'voxelsize_mm': 1,
                        'datapath': None,
                        'data3d': data3d
                        }

            mindatap.update(metadata)
            self.import_dataplus(mindatap)

            # self.iparams['series_number'] = self.metadata['series_number']
            # self.iparams['datapath'] = self.metadata['datapath']
        # self.import_dataplus()

    # def importDataPlus(self, datap):
    #    """
    #    Function for input data
    #    """
    #    self.data3d = datap['data3d']
    #    self.crinfo = datap['crinfo']
    #    self.segmentation = datap['segmentation']
    #    self.slab = datap['slab']
    #    self.voxelsize_mm = datap['voxelsize_mm']
    #    self.orig_shape = datap['orig_shape']
    #    self.seeds = datap[
    #        'processing_information']['organ_segmentation']['seeds']
    def update_parameters_based_on_label(self, label):
        self.update_parameters(self.segmentation_alternative_params[label])

    def update_parameters(self, params):
        if 'segmodelparams' in params.keys():
            self.segmodelparams = params['segmodelparams']
            logger.debug('segmodelparams updated')
        if 'output_label' in params.keys():
            self.output_label = params['output_label']
            logger.debug('output_label updated')
        if 'working_voxelsize_mm' in params.keys():
            self.input_wvx_size = copy.copy(params['working_voxelsize_mm'])
            self.working_voxelsize_mm = params['working_voxelsize_mm']
            vx_size = self.working_voxelsize_mm

            if np.isscalar(vx_size):
                vx_size = ([vx_size] * 3)

            vx_size = np.array(vx_size).astype(float)

            self.working_voxelsize_mm = vx_size
            logger.debug('working_voxelsize_mm updated')
        if 'smoothing_mm' in params.keys():
            self.smoothing_mm = params['smoothing_mm']
            logger.debug('smoothing_mm updated')
        if 'seg_postproc_pars' in params.keys():
            self.seg_postproc_pars = params['seg_postproc_pars']
            logger.debug('seg_postproc_pars updated')


    def run_sss(self):
        sseg = sss.SupportStructureSegmentation(
            data3d=self.data3d,
            voxelsize_mm=self.voxelsize_mm,
        )
        sseg.run()

        # sseg.bone_segmentation()
        # sseg.lungs_segmentation()
        # sseg.heart_segmentation()

        # TODO remove hack - force remove number 1 from segmentation
        # this sould be fixed in sss
        sseg.segmentation[sseg.segmentation == 1] = 0
        self.segmentation = sseg.segmentation
        self.slab = sseg.slab

    def __clean_oseg_input_params(self, oseg_params):
        """
        Used for storing input params of organ segmentation. Big data are not
        stored due to big  memory usage.
        """
        oseg_params['data3d'] = None
        oseg_params['segmentation'] = None
        oseg_params.pop('self')
        oseg_params.pop('pycut')
        return oseg_params

    def process_wvx_size_mm(self, metadata):
        """This function does something.

            Args:
                name (str):  The name to use.

            Kwargs:
                state (bool): Current state to be in.

        """

        # vx_size = self.working_voxelsize_mm
        vx_size = self.input_wvx_size
        if vx_size == 'orig':
            vx_size = metadata['voxelsize_mm']

        elif vx_size == 'orig*2':
            vx_size = np.array(metadata['voxelsize_mm']) * 2

        elif vx_size == 'orig*4':
            vx_size = np.array(metadata['voxelsize_mm']) * 4

        if np.isscalar(vx_size):
            vx_size = ([vx_size] * 3)

        vx_size = np.array(vx_size).astype(float)

        # if np.isscalar(vx_sizey):
        #     vx_size = (np.ones([3]) *vx_size).astype(float)

        # self.iparams['working_voxelsize_mm'] = vx_size
        self.working_voxelsize_mm = vx_size
        # return vx_size

    def __volume_blowup_criterial_function(self, threshold, wanted_volume,
                                           segmentation_smooth
                                           ):

        segm = (1.0 * segmentation_smooth > threshold).astype(np.int8)
        vol2 = np.sum(segm)
        criterium = (wanted_volume - vol2) ** 2
        return criterium

    def sliver_compare_with_other_volume_from_file(self, filepath):
        reader = datareader.DataReader()
        segmentation_datap = reader.Get3DData(filepath, dataplus_format=True)
        evaluation = self.sliver_compare_with_other_volume(segmentation_datap)
        return evaluation

    def sliver_compare_with_other_volume(self, segmentation_datap):
        """
        Compares actual Lisa data with other which are given by
        segmentation_datap. That means
        segmentation_datap = {
            'segmentation': 3d np.array,
            'crinfo': information about crop (optional)
            }

        """
        # if there is no segmentation, data can be stored in data3d. It is the
        # way how are data stored in sliver.
        if 'segmentation' in segmentation_datap.keys():
            segm_key = 'segmentation'
        else:
            segm_key = 'data3d'
        if 'crinfo' in segmentation_datap.keys():
            data3d_segmentation = qmisc.uncrop(
                segmentation_datap[segm_key],
                segmentation_datap['crinfo'],
                self.orig_shape)
        else:
            data3d_segmentation = segmentation_datap[segm_key]
        pass

        # now we can uncrop actual Lisa data
        data3d_segmentation_actual = qmisc.uncrop(
            self.segmentation,
            self.crinfo,
            self.orig_shape)

        label1 = 1
        label2 = 1
# TODO make GUI in Qt
        from PyQt4.QtCore import pyqtRemoveInputHook
        pyqtRemoveInputHook()
        print 'unique data1 ', np.unique(data3d_segmentation_actual)
        print 'unique data2 ', np.unique(data3d_segmentation)
        print "set label1 and label2"
        print "then press 'c' and 'Enter'"
        import ipdb; ipdb.set_trace() #  noqa BREAKPOINT

        evaluation = volumetry_evaluation.compare_volumes_sliver(
            data3d_segmentation_actual == label1,
            data3d_segmentation == label2,
            self.voxelsize_mm
            )
        # score = volumetry_evaluation.sliver_score_one_couple(evaluation)
        segdiff = qmisc.crop(
            ((data3d_segmentation) - data3d_segmentation_actual),
            self.crinfo)
        return evaluation, segdiff

    def segm_smoothing(self, sigma_mm):
        """
        Shape of output segmentation is smoothed with gaussian filter.

        Sigma is computed in mm

        """
        # import scipy.ndimage
        sigma = float(sigma_mm) / np.array(self.voxelsize_mm)

        # print sigma
        # from PyQt4.QtCore import pyqtRemoveInputHook
        # pyqtRemoveInputHook()
        vol1 = np.sum(self.segmentation)
        wvol = vol1 * self.volume_blowup
        logger.debug('unique segm ' + str(np.unique(self.segmentation)))
        segsmooth = scipy.ndimage.filters.gaussian_filter(
            self.segmentation.astype(np.float32), sigma)
        logger.debug('unique segsmooth ' + str(np.unique(segsmooth)))
        # import ipdb; ipdb.set_trace()
        # import pdb; pdb.set_trace()
        # pyed = sed3.sed3(self.orig_scale_segmentation)
        # pyed.show()
        logger.debug('wanted volume ' + str(wvol))
        logger.debug('sigma ' + str(sigma))

        critf = lambda x: self.__volume_blowup_criterial_function(
            x, wvol, segsmooth)

        thr = scipy.optimize.fmin(critf, x0=0.5, disp=False)[0]
        logger.debug('optimal threshold ' + str(thr))
        logger.debug('segsmooth ' + str(np.nonzero(segsmooth)))

        self.segmentation = (1.0 *
                             (segsmooth > thr)  # self.volume_blowup)
                             ).astype(np.int8)
        vol2 = np.sum(self.segmentation)
        logger.debug("volume ratio " + str(vol2 / float(vol1)))
        # import ipdb; ipdb.set_trace()

    def import_segmentation_from_file(self, filepath):
        """
        Loads data from file. Expected are uncropped data.
        """
        # logger.debug("import segmentation from file")
        # logger.debug(str(self.crinfo))
        reader = datareader.DataReader()
        datap = reader.Get3DData(filepath, dataplus_format=True)
        segmentation = datap['data3d']
        segmentation = qmisc.crop(segmentation, self.crinfo)
        logger.debug(str(segmentation.shape))
        self.segmentation = segmentation

    def import_dataplus(self, dataplus):
        datap = {
            'dcmfilelist': None,
        }
        datap.update(dataplus)

        dpkeys = datap.keys()
        self.data3d = datap['data3d']

        if self.roi is not None:
            self.crop(self.roi)

        self.voxelsize_mm = np.array(datap['voxelsize_mm'])
        self.process_wvx_size_mm(datap)
        self.autocrop_margin = self.autocrop_margin_mm / self.voxelsize_mm
        if 'orig_shape' in dpkeys:
            self.orig_shape = datap['orig_shape']
        else:
            self.orig_shape = self.data3d.shape

        if 'crinfo' in dpkeys:
            self.crinfo = datap['crinfo']
        if 'slab' in dpkeys:
            self.slab = datap['slab']

        if ('segmentation' in dpkeys) and datap['segmentation'] is not None:
            self.segmentation = datap['segmentation']
        else:
            self.segmentation = np.zeros(self.data3d.shape, dtype=np.int8)
        if 'vessel_tree' in dpkeys:
            self.vessel_tree = datap['vessel_tree']

        if ('apriori' in dpkeys) and datap['apriori'] is not None:
            self.apriori= datap['apriori']
        else:
            self.apriori = None

        self.dcmfilelist = datap['dcmfilelist']

        self.segparams['pairwise_alpha'] = \
            self.segparams['pairwise_alpha_per_mm2'] / \
            np.mean(self.working_voxelsize_mm)

        self.__import_dataplus_seeds(datap)

        # chci, abych nepřepisoval uložené seedy
        if self.after_load_processing['run_automatic_liver_seeds']:
            if self.seeds is None or (self.seeds == 0).all():
                self.automatic_liver_seeds()

        # try read prev information about time processing
        try:
            time_prev = datap['processing_information']['processing_time']
            self.processing_time = time_prev
            self.time_start = datetime.datetime.now() - time_prev
        except:
            self.time_start = datetime.datetime.now()

    def __import_dataplus_seeds(self, datap):
        """

        :type self: seeds are changed
        """
        try:
            self.seeds = datap['processing_information'][
                'organ_segmentation']['seeds']
        except:
            logger.info('seeds not found in dataplus')
            # if dicomdir is readed after something with seeds, seeds needs to be reseted
            # self.seeds = None

        # for each mm on boundary there will be sum of penalty equal 10

        if self.seeds is None:
            logger.debug("Seeds are generated")
            self.seeds = np.zeros(self.data3d.shape, dtype=np.int8)
        logger.debug("unique seeds labels " + str(np.unique(self.seeds)))
        logger.info('dir ' + str(self.datapath) + ", series_number" +
                    str(datap['series_number']) + 'voxelsize_mm' +
                    str(self.voxelsize_mm))

    def crop(self, tmpcrinfo):
        """
        Function makes crop of 3d data and seeds and stores it in crinfo.

        tmpcrinfo: temporary crop information

        """
        # print ('sedds ', str(self.seeds.shape), ' se ',
        #       str(self.segmentation.shape), ' d3d ', str(self.data3d.shape))
        self.data3d = qmisc.crop(self.data3d, tmpcrinfo)
# No, size of seeds should be same as data3d
        if self.seeds is not None:
            self.seeds = qmisc.crop(self.seeds, tmpcrinfo)

        if self.segmentation is not None:
            self.segmentation = qmisc.crop(self.segmentation, tmpcrinfo)

        self.crinfo = qmisc.combinecrinfo(self.crinfo, tmpcrinfo)
        logger.debug("crinfo " + str(self.crinfo))

        # print '----sedds ', self.seeds.shape, ' se ',
# self.segmentation.shape,\
        #        ' d3d ', self.data3d.shape

    def _interactivity_begin(self):
        from pysegbase import pycut
        logger.debug('_interactivity_begin()')
        # TODO make copy and work with it
        # TODO really make the copy and work with it

        data3d_tmp = self.data3d
        if self.seg_preproc_pars['use_automatic_segmentation']:
            data3d_tmp = self.data3d.copy()
            data3d_tmp[(self.segmentation > 0) & (self.segmentation != self.output_label)] = -1000

        # print 'zoom ', self.zoom
        # print 'svs_mm ', self.working_voxelsize_mm
        self.zoom = self.voxelsize_mm / (1.0 * self.working_voxelsize_mm)
        data3d_res = scipy.ndimage.zoom(
            self.data3d,
            self.zoom,
            mode='nearest',
            order=1
        ).astype(np.int16)
        # data3d_res = data3d_res.astype(np.int16)

        logger.debug('pycut segparams ' + str(self.segparams) +
                     '\nmodelparams ' + str(self.segmodelparams)
                     )
        # insert feature function instead of string description
        import liver_model
        self.segmodelparams = liver_model.add_fv_extern_into_modelparams(self.segmodelparams)

        if 'method' not in self.segparams.keys() or\
                self.segparams['method'] in pycut.methods:
            igc = pycut.ImageGraphCut(
                # self.data3d,
                data3d_res,
                segparams=self.segparams,
                voxelsize=self.working_voxelsize_mm,
                modelparams=self.segmodelparams,
                volume_unit='ml'
                # oxelsize=self.voxelsize_mm
            )
        # elif self.segparams['method'] == '':
        else:
            import liver_segmentation
            igc = liver_segmentation.LiverSegmentation(
                data3d_res,
                voxelsize_mm=self.working_voxelsize_mm,
                segparams=self.segparams
            )
        if self.apriori is not None:
            apriori_res = misc.resize_to_shape(
                    # seeds_res = scipy.ndimage.zoom(
                    self.apriori,
                    data3d_res.shape,
            )
            igc.apriori = apriori_res

        # igc.modelparams = self.segmodelparams
# @TODO uncomment this for kernel model
#        igc.modelparams = {
#            'type': 'kernel',
#            'params': {}
#        }
        # if self.iparams['seeds'] is not None:
        if self.seeds is not None:
            seeds_res = misc.resize_to_shape(
            # seeds_res = scipy.ndimage.zoom(
                self.seeds,
                data3d_res.shape,
                mode='nearest',
                order=0
            )
            seeds_res = seeds_res.astype(np.int8)
            igc.set_seeds(seeds_res)

        # tohle je tu pro to, aby bylo možné přidávat nově objevené segmentace k těm starým
        # jinak jsou stará data přepsána
        if self.segmentation is not None:
            self.segmentation_prev = copy.copy(self.segmentation)
        else:
            self.segmentation_prev = None

        return igc
    def sync_lisa_data(self, username, password, host="147.228.47.162", callback=printTotals):
        self.create_lisa_data_dir_tree()


        import sftpsync
        import paramiko

        paramiko.util.log_to_file('paramiko.log')
        sftp = sftpsync.Sftp(host=host, username=username, password=password)
        localfrom = self._output_datapath_from_server
        localto = self._output_datapath_to_server
        remotefrom = "from_server/"
        remoteto = "to_server/"

        exclude = []

        logger.info("Download started")
        sftp.sync(remotefrom, localfrom, download=True, exclude=exclude, delete=False, callback=callback)
        logger.info("Download finished")
        logger.info("Upload started")
        sftp.sync(localto, remoteto, download=False, exclude=exclude, delete=False, callback=callback)
        logger.info("Upload finished")


    def __resize_to_orig(self, igc_seeds):
        # @TODO remove old code in except part
        self.segmentation = misc.resize_to_shape(
            self.segmentation,
            self.data3d.shape,
            self.zoom
        )
        self.seeds = misc.resize_to_shape(
            igc_seeds,
            self.data3d.shape,
            self.zoom
        ).astype(np.uint8)

#         try:
#             # rint 'pred vyjimkou'
#             # aise Exception ('test without skimage')
#             # rint 'za vyjimkou'
#             import skimage
#             import skimage.transform
# # Now we need reshape  seeds and segmentation to original size
#
#             segm_orig_scale = skimage.transform.resize(
#                 self.segmentation, self.data3d.shape, order=0,
#                 preserve_range=True
#             )
#
#             seeds = skimage.transform.resize(
#                 igc_seeds, self.data3d.shape, order=0,
#                 preserve_range=True
#             )
#
#             # self.segmentation = segm_orig_scale
#             self.seeds = seeds
#             logger.debug('resize to orig with skimage')
#         except:
#
#             segm_orig_scale = scipy.ndimage.zoom(
#                 self.segmentation,
#                 1.0 / self.zoom,
#                 mode='nearest',
#                 order=0
#             ).astype(np.int8)
#             seeds = scipy.ndimage.zoom(
#                 igc_seeds,
#                 1.0 / self.zoom,
#                 mode='nearest',
#                 order=0
#             )
#             logger.debug('resize to orig with scipy.ndimage')
#
# # @TODO odstranit hack pro oříznutí na stejnou velikost
# # v podstatě je to vyřešeno, ale nechalo by se to dělat elegantněji v zoom
# # tam je bohužel patrně bug
#             # rint 'd3d ', self.data3d.shape
#             # rint 's orig scale shape ', segm_orig_scale.shape
#             shp = [
#                 np.min([segm_orig_scale.shape[0], self.data3d.shape[0]]),
#                 np.min([segm_orig_scale.shape[1], self.data3d.shape[1]]),
#                 np.min([segm_orig_scale.shape[2], self.data3d.shape[2]]),
#             ]
#             # elf.data3d = self.data3d[0:shp[0], 0:shp[1], 0:shp[2]]
#             # mport ipdb; ipdb.set_trace() # BREAKPOINT
#
#             self.segmentation = np.zeros(self.data3d.shape, dtype=np.int8)
#             self.segmentation[
#                 0:shp[0],
#                 0:shp[1],
#                 0:shp[2]] = segm_orig_scale[0:shp[0], 0:shp[1], 0:shp[2]]
#
#             del segm_orig_scale
#
#             self.seeds[
#                 0:shp[0],
#                 0:shp[1],
#                 0:shp[2]] = seeds[0:shp[0], 0:shp[1], 0:shp[2]]

    def _interactivity_end(self, igc):
        """
        This is called after processing step. All data are rescaled to original
        resolution.
        """
        logger.debug('_interactivity_end()')

        self.__resize_to_orig(igc.seeds)
        self.organ_interactivity_counter = igc.interactivity_counter
        logger.debug("org inter counter " +
                     str(self.organ_interactivity_counter))
        logger.debug('nonzero segm ' + str(np.nonzero(self.segmentation)))
        # if False:
        if False:
            # TODO dodělat postprocessing PV
            import segmentation
            outputSegmentation = segmentation.vesselSegmentation(  # noqa
                self.data3d,
                self.segmentation,
                threshold=-1,
                inputSigma=0.15,
                dilationIterations=10,
                nObj=1,
                biggestObjects=False,
                seeds=(self.segmentation > 0).astype(np.int8),
                useSeedsOfCompactObjects=True,
                interactivity=True,
                binaryClosingIterations=2,
                binaryOpeningIterations=0)

        self._segmentation_postprocessing()

        # rint 'autocrop', self.autocrop
        if self.autocrop is True:
            # rint
            # mport pdb; pdb.set_trace()

            tmpcrinfo = qmisc.crinfo_from_specific_data(
                self.segmentation,
                self.autocrop_margin)

            self.crop(tmpcrinfo)

        if self.texture_analysis not in (None, False):
            import texture_analysis
            # doplnit nějaký kód, parametry atd
            # elf.orig_scale_segmentation =
            # texture_analysis.segmentation(self.data3d,
            # self.orig_scale_segmentation, params = self.texture_analysis)
            self.segmentation = texture_analysis.segmentation(
                self.data3d,
                self.segmentation,
                self.voxelsize_mm
            )

        # set label number
# !! pomaly!!!
# @TODO make faster
        # spojení staré a nové segmentace
        if self.segmentation_prev is None:
            # pokud neznáme žádnou předchozí segmentaci, tak se chováme jako dříve
            self.segmentation[self.segmentation == 1] = self.output_label
        else:
            # remove old pixels for this label
            self.segmentation_prev[self.segmentation_prev == self.output_label] = 0
            # set new labels
            self.segmentation_prev[self.segmentation == 1] = self.output_label

            # clean up

            self.segmentation = self.segmentation_prev
            self.segmentation_prev = None

#
        logger.debug('self.slab')
        logger.debug(str(self.slab))
        self.processing_time = (
            datetime.datetime.now() - self.time_start).total_seconds()

        logger.debug('processing_time = ' + str(self.processing_time))

    def _segmentation_postprocessing(self):
        """
        :segmentation_smoothing:
        """
        logger.debug(str(self.seg_postproc_pars))
        if self.seg_postproc_pars['segmentation_smoothing']:
            # if self.segmentation_smoothing:
            self.segm_smoothing(self.seg_postproc_pars['smoothing_mm'])

        if self.seg_postproc_pars['snakes']:
            import morphsnakes as ms
            logger.debug('Making snakes')
            if self.seg_postproc_pars['snakes_method'] is 'ACWE':
                method = ms.MorphACWE
            elif self.seg_postproc_pars['snakes_method'] is 'GAC':
                method = ms.MorphGAC
            else:
                logger.error('Unknown snake method')
                return

            sp = self.seg_postproc_pars['snakes_params']
            if 'seeds' in sp.keys() and sp['seeds'] is True:
                sp['seeds'] = self.seeds

            logger.debug('snakes')
            d3d = qmisc.resize_to_mm(
                    self.data3d, 
                    self.voxelsize_mm, 
                    self.seg_postproc_pars['postproc_working_voxelsize'])
            segw = qmisc.resize_to_mm(
                    self.segmentation, 
                    self.voxelsize_mm, 
                    self.seg_postproc_pars['postproc_working_voxelsize'])
            macwe = method(
                d3d,
                # self.data3d,
                **self.seg_postproc_pars['snakes_params']
            )
            macwe.levelset = (
                # self.segmentation == self.slab['liver']
                segw == self.slab['liver']
            ).astype(np.uint8)
            macwe.run(self.seg_postproc_pars['snakes_niter'])
            seg = qmisc.resize_to_shape(macwe.levelset, self.data3d.shape)
# for debug visualization preprocessing use fallowing line
            # self.segmentation[seg == 1] += 1
            self.segmentation[seg == 1] = self.slab['liver']
            logger.debug('postprocessing with snakes finished')

#    def interactivity(self, min_val=800, max_val=1300):
# @TODO generovat QApplication
    def interactivity(self, min_val=None, max_val=None):
        try:
            from pysegbase.seed_editor_qt import QTSeedEditor
        except:
            logger.warning("Deprecated of pyseg_base as submodule")
            from seed_editor_qt import QTSeedEditor
        import_gui()
        logger.debug('interactivity')
        # if self.edit_data:
        #     self.data3d = self.data_editor(self.data3d)

        igc = self._interactivity_begin()
        # from PyQt4.QtCore import pyqtRemoveInputHook
        # pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT

        pyed = QTSeedEditor(igc.img,
                            seeds=igc.seeds,
                            modeFun=igc.interactivity_loop,
                            voxelSize=igc.voxelsize,
                            volume_unit='ml')

        # set window
        if min_val is None:
            min_val = np.min(self.data3d)

        if max_val is None:
            max_val = np.max(self.data3d)

        window_c = ((max_val + min_val) / 2)
        window_w = (max_val - min_val)

        pyed.changeC(window_c)
        pyed.changeW(window_w)

        pyed.exec_()

# @TODO někde v igc.interactivity() dochází k přehození nul za jedničy,
# tady se to řeší hackem
        if igc.segmentation is not None:
            self.segmentation = (igc.segmentation == 0).astype(np.int8)
        self._interactivity_end(igc)

    def ninteractivity(self):
        from pysegbase import pycut
        """Function for automatic (noninteractiv) mode."""
        # mport pdb; pdb.set_trace()
        igc = self._interactivity_begin()
        # gc.interactivity()
        # igc.make_gc()
        igc.run()
        if 'method' not in self.segparams.keys() or\
\
                self.segparams['method'] in pycut.methods:
            logger.debug('ninteractivity seg method GC')
            self.segmentation = (igc.segmentation == 0).astype(np.int8)
        else:
            logger.debug('ninteractivity seg method other')
            self.segmentation = np.asarray(igc.segmentation, dtype=np.int8)
        self._interactivity_end(igc)

    def export(self):
        slab = {}
        slab['none'] = 0
        slab['liver'] = 1
        slab['lesions'] = 6
        slab.update(self.slab)

        data = {}
        data['version'] = (1, 0, 1)
        data['data3d'] = self.data3d
        data['crinfo'] = self.crinfo
        data['segmentation'] = self.segmentation
        data['apriori'] = self.apriori
        data['slab'] = slab
        data['voxelsize_mm'] = self.voxelsize_mm
        data['orig_shape'] = self.orig_shape
        data['vessel_tree'] = self.vessel_tree
        processing_information = {
            'organ_segmentation': {
                'processing_time': self.processing_time,
                'time_start': str(self.time_start),
                'oseg_input_params': self.oseg_input_params,
                'organ_interactivity_counter':
                self.organ_interactivity_counter,
                'seeds': self.seeds  # qmisc.SparseMatrix(self.seeds)
            }
        }
        data['processing_information'] = processing_information
        # from PyQt4 import QtCore
        # QtCore.pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace()
# TODO add dcmfilelist
        logger.debug("export()")
        # ogger.debug(str(data))
        logger.debug("org int ctr " + str(self.organ_interactivity_counter))
        # ata["metadata"] = self.metadata
        # mport pdb; pdb.set_trace()
        return data

    # def get_iparams(self):
    #     self.iparams['seeds'] = qmisc.SparseMatrix(self.iparams['seeds'])

    #     return self.iparams

    def automatic_liver_seeds(self):
        seeds, likdif = liver_seeds.automatic_liver_seeds(self.data3d, self.seeds, self.voxelsize_mm)
        # přenastavíme na čísla mezi nulou a jedničkou, druhá konstanta je nastavena empiricky
        self.apriori = boltzman(likdif, 0, 200).astype(np.float16)

    def add_seeds_mm(self, z_mm, x_mm, y_mm, label, radius, width=1):
        """
        Function add circle seeds to one slice with defined radius.

        It is possible set more seeds on one slice with one dimension

        x_mm, y_mm coordinates of circle in mm. It may be array.
        z_mm = slice coordinates  in mm. It may be array
        :param label: one number. 1 is object seed, 2 is background seed
        :param radius: is radius of circle in mm
        :param width: makes circle with defined width (repeat circle every milimeter)

        """
        import data_manipulation

        data_manipulation.add_seeds_mm(
            self.seeds, self.voxelsize_mm,
            z_mm, x_mm, y_mm,
            label,
            radius, width
        )

    def lesionsLocalization(self):
        """ Localization of lession """
        import lesions
        tumory = lesions.Lesions()
        # tumory.overlay_test()
        data = self.export()
        tumory.import_data(data)
        tumory.run_gui()
        # tumory.automatic_localization()

        self.segmentation = tumory.segmentation

    def portalVeinSegmentation(self, **inparams):
        """
        Segmentation of vein in specified volume. It is given by label "liver".
        Usualy it is number 1. If there is no specified volume all image is
        used.

        If this function is used repeatedly (if there is some segmentation in
        this image) all segmentation labeled as 'porta' is removed and setted
        to 'liver' before processing.

        You can use additional parameters from vesselSegmentation()
        For example interactivity=False, biggestObjects=True, ...
        """

        from imtools import segmentation
        logger.info('segmentation max label ' + str(np.max(self.segmentation)))
        # if there is no organ segmentation, use all image
        slab = {'porta': 2}
        slab.update(self.slab)
        logger.debug('self.slab')
        logger.debug(str(self.slab))
        self.slab = slab

        # if there is no liver segmentation, use whole image
        if np.max(self.segmentation) == 0:
            self.segmentation = self.segmentation + 1

        # remove prev segmentation
        self.segmentation[self.segmentation == slab['porta']] = slab['liver']

        params = {
            'threshold': -1,
            'inputSigma': 0.15,
            'dilationIterations': 10,
            'nObj': 1,
            'biggestObjects': False,
            'useSeedsOfCompactObjects': True,
            'interactivity': True,
            'binaryClosingIterations': 2,
            'binaryOpeningIterations': 0
        }
        params.update(inparams)
        outputSegmentation = segmentation.vesselSegmentation(
            self.data3d,
            self.segmentation,
            **params
            # threshold=-1,
            # inputSigma=0.15,
            # dilationIterations=10,
            # nObj=1,
            # biggestObjects=False,
            # useSeedsOfCompactObjects=True,
            # # useSeedsOfCompactObjects=False,
            # interactivity=True,
            # binaryClosingIterations=2,
            # binaryOpeningIterations=0
        )
        # rom PyQt4.QtCore import pyqtRemoveInputHook
        # yqtRemoveInputHook()
        # mport ipdb; ipdb.set_trace() # BREAKPOINT
        self.segmentation[outputSegmentation == 1] = slab['porta']

        # self.__vesselTree(outputSegmentation, 'porta')

    def saveVesselTree(self, textLabel):
        """
        textLabel: 'porta' or 'hepatic_veins'
        """
        self.__vesselTree(
            self.segmentation == self.slab[textLabel],
            textLabel
        )

    def __vesselTree(self, binaryData3d, textLabel):
        import skelet3d
        from skelet3d import skeleton_analyser  # histology_analyser as skan
        data3d_thr = (binaryData3d > 0).astype(np.int8)
        data3d_skel = skelet3d.skelet3d(data3d_thr)

        skan = skeleton_analyser.SkeletonAnalyser(
            data3d_skel,
            volume_data=data3d_thr,
            voxelsize_mm=self.voxelsize_mm)
        stats = skan.skeleton_analysis(guiUpdateFunction=None)

        if 'Graph' not in self.vessel_tree.keys():
            self.vessel_tree['voxelsize_mm'] = self.voxelsize_mm
            self.vessel_tree['Graph'] = {}

        self.vessel_tree['Graph'][textLabel] = stats
        # print sa.stats
        logger.debug('save vessel tree to file')
        fn = self.get_standard_ouptut_filename(filetype='yaml', suffix='vessel_tree')
# save all skeletons to one special file
        misc.obj_to_file(self.vessel_tree, fn, filetype='yaml')
        logger.debug('save vessel tree to file - finished')
        # generate vtk file
        logger.debug('start to generate vtk file from vessel_tree')
        import imtools.gen_vtk_tree
        fn = self.get_standard_ouptut_filename(filetype='vtk', suffix='vt-' + textLabel)
        imtools.gen_vtk_tree.vt2vtk_file(self.vessel_tree, fn, text_label=textLabel)
        logger.debug('generating vtk file from vessel_tree finished')

    def hepaticVeinsSegmentation(self):

        from imtools import segmentation
        outputSegmentation = segmentation.vesselSegmentation(
            self.data3d,
            self.segmentation,
            threshold=-1,
            inputSigma=0.15,
            dilationIterations=10,
            nObj=1,
            biggestObjects=False,
            useSeedsOfCompactObjects=True,
            interactivity=True,
            binaryClosingIterations=2,
            binaryOpeningIterations=0)
        slab = {'hepatic_veins': 3}
        slab.update(self.slab)
        # rom PyQt4.QtCore import pyqtRemoveInputHook
        # yqtRemoveInputHook()
        # mport ipdb; ipdb.set_trace() # BREAKPOINT
        self.slab = slab
        self.segmentation[outputSegmentation == 1] = slab['hepatic_veins']

# skeletonizace
#         self.__vesselTree(outputSegmentation, 'hepatic_veins')

    def get_segmented_volume_size_mm3(self):
        """Compute segmented volume in mm3, based on subsampeled data."""

        voxelvolume_mm3 = np.prod(self.voxelsize_mm)
        volume_mm3 = np.sum(self.segmentation > 0) * voxelvolume_mm3
        return volume_mm3

    def get_standard_ouptut_filename(self, filetype=None, suffix=''):
        """
        It can be settet filename, or filename end with suffix.
        """
        if filetype is None:
            filetype = self.save_filetype

        output_dir = self.output_datapath

        pth, filename = op.split(op.normpath(self.datapath))
        filename += "-" + self.experiment_caption
#        if savestring in ['a', 'A']:
# save renamed file too
        filename = '' + filename + suffix + '.' + filetype
        filepath = op.join(output_dir, filename)
        filepath = misc.suggest_filename(filepath)

        return filepath

    def save_outputs(self, filepath=None):

        # savestring_qt, ok = QInputDialog.getText(
        #     None,
        #     "Save",
        #     'Save output data? Yes/No/All with input data (y/n/a):',
        #     text="a"
        #     )

        # savestring = str(savestring_qt)

        #    if savestring in ['Y', 'y', 'a', 'A']:

        data = self.export()
        data['version'] = self.version  # qmisc.getVersionString()
        data['experiment_caption'] = self.experiment_caption
        data['lisa_operator_identifier'] = self.lisa_operator_identifier
#       data['organ_interactivity_counter'] = self.organ_interactivity_counter
# save renamed file too
#         pth, filename = op.split(op.normpath(self.datapath))
#         filename += "-" + self.experiment_caption
# #        if savestring in ['a', 'A']:
#         filepath = 'org-' + filename + '.' + self.save_filetype
#         # rint filepath
#         # rint 'op ', op
        self.create_lisa_data_dir_tree()

        if filepath is None:
            filepath = self.get_standard_ouptut_filename()
        # import ipdb; ipdb.set_trace()
        import io3d
        io3d.write(data, filepath)
        # from PyQt4 import QtCore; QtCore.pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace()
        # misc.obj_to_file(data, filepath, filetype=self.save_filetype)

        # filepath2 = 'organ_last.' + self.save_filetype
        # filepath2 = op.join(odp, filepath2)
        # # ilepath = misc.suggest_filename(filepath)
        # misc.obj_to_file(data, filepath2, filetype=self.save_filetype)
# save to mat

#        iparams = self.get_iparams()
        # filepath = 'organ_iparams.pklz'
        # filepath = op.join(odp, filepath)
        # misc.obj_to_file(iparams, filepath, filetype='pklz')

        # f savestring in ['a', 'A']:
    def create_lisa_data_dir_tree(self):
        # used for server sync
        self._output_datapath_from_server = op.join(self.output_datapath, 'sync', self.sftp_username ,"from_server" )
        # used for server sync
        self._output_datapath_to_server = op.join(self.output_datapath, 'sync', self.sftp_username, "to_server" )
        odp = self.output_datapath
        if not op.exists(odp):
            os.makedirs(odp)
        odp = self._output_datapath_from_server
        if not op.exists(odp):
            os.makedirs(odp)
        odp = self._output_datapath_to_server
        if not op.exists(odp):
            os.makedirs(odp)

    def rotate(self, angle, axes):
        self.data3d = scipy.ndimage.interpolation.rotate(self.data3d, angle, axes)
        self.segmentation = scipy.ndimage.interpolation.rotate(self.segmentation, angle, axes)
        self.seeds = scipy.ndimage.interpolation.rotate(self.seeds, angle, axes)


    def random_rotate(self):
        angle = np.random.rand() * 360
        self.rotate(angle, (1, 2))

    def save_input_dcm(self, filename):
        # TODO add
        logger.debug('save dcm')
        dw = datawriter.DataWriter()
        dw.Write3DData(self.data3d, filename, filetype='dcm',
                       metadata={'voxelsize_mm': self.voxelsize_mm})

    def save_outputs_dcm(self, filename):
        # TODO add
        logger.debug('save dcm')
        dw = datawriter.DataWriter()
        dw.Write3DData(self.segmentation.astype(np.int16), filename,
                       filetype='dcm',
                       metadata={'voxelsize_mm': self.voxelsize_mm})

    def save_outputs_dcm_overlay(self):
        # TODO add
        logger.debug('save dcm')
        from PyQt4.QtCore import pyqtRemoveInputHook
        pyqtRemoveInputHook()
        # mport ipdb; ipdb.set_trace() # BREAKPOINT
        odp = self.output_datapath
        pth, filename = op.split(op.normpath(self.datapath))
        filename += "-" + self.experiment_caption
        # f savestring in ['ad']:
        #       save to DICOM
        filepath = 'dicom-' + filename
        filepath = os.path.join(odp, filepath)
        filepath = misc.suggest_filename(filepath)
        output_dicom_dir = filepath
        data = self.export()
        # mport ipdb; ipdb.set_trace()  # BREAKPOINT
        overlays = {
            3:
            (data['segmentation'] == self.output_label).astype(np.int8)
        }
        if self.dcmfilelist is not None:
            datawriter.saveOverlayToDicomCopy(
                self.dcmfilelist,
                output_dicom_dir, overlays,
                data['crinfo'], data['orig_shape'])


def logger_init():  # pragma: no cover
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.ERROR)
    formatter = logging.Formatter(
        '%(name)s - %(levelname)s - %(message)s'
    )
    ch.setFormatter(formatter)
    logger.addHandler(ch)

    fformatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(funcName)s - %(lineno)d - %(levelname)s - %(message)s'
    )
    logfile = "lisa.log"
    if op.exists(op.expanduser("~/lisa_data/")):
        logfile = op.expanduser("~/lisa_data/lisa.log")
    fh = logging.FileHandler(logfile)
    fh.setFormatter(fformatter)
    fh.setLevel(logging.DEBUG)
    logger.addHandler(fh)

    logger.debug('logger started')

    return ch, fh


def lisa_config_init():
    """
    Generate default config from function paramteres.
    Specific config given by command line argument is implemented in
    parser_init() function.
    """
    # read confguraton from file, use default values from OrganSegmentation
    cfg = config.get_default_function_config(OrganSegmentation.__init__)

    # for parameters without support in OrganSegmentation or to overpower
    # default OrganSegmentation values use cfgplus
    cfgplus = {
        'datapath': None,
        'viewermax': 225,
        'viewermin': -125,
        'output_datapath': os.path.expanduser("~/lisa_data"),
        'input_datapath_start': os.path.expanduser("~/lisa_data")
        # config_version':[1,1]
    }

    cfg.update(cfgplus)
    # now is in cfg default values

    # cfg = config.get_config("organ_segmentation.config", cfg)
    cfg.update(config_default.CONFIG_DEFAULT)
    user_config_path = os.path.join(cfg['output_datapath'],
                                    "organ_segmentation.config")
    config.check_config_version_and_remove_old_records(
        user_config_path, version=config_version,
        records_to_save=['experiment_caption', 'lisa_operator_identifier'])
    # read user defined config in user data
    cfg = config.get_config(user_config_path, cfg)
    return cfg


def parser_init(cfg):  # pragma: no cover

    # input parser
    conf_parser = argparse.ArgumentParser(
        # Turn off help, so we print all options in response to -h
        add_help=False
    )
    conf_parser.add_argument(
        '-cf', '--configfile', default=None,
        help="Use another config. It is loaded after default \
config and user config.")
# Read alternative config file. First is loaded default config. Then user
# config in lisa_data directory. After that is readed config defined by
# --configfile parameter
    knownargs, unknownargs = conf_parser.parse_known_args()

    parser = argparse.ArgumentParser(
        # Inherit options from config_parser
        parents=[conf_parser],
        # print script description with -h/--help
        description=__doc__,
        # Don't mess with format of description
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    if knownargs.configfile is not None:
        cfg = config.get_config(knownargs.configfile, cfg)

    parser.add_argument('-dd', '--datapath',
                        default=cfg["datapath"],
                        help='path to data dir')
    parser.add_argument('-d', '--debug', action='store_true',
                        help='run in debug mode')
    parser.add_argument(
        '-vs', '--working_voxelsize_mm',
        default=cfg["working_voxelsize_mm"],
        type=eval,  # type=str,
        help='Insert working voxelsize. It can be number or \
        array of three numbers. It is possible use original \n \
        resolution or half of original resolution. \n \
        -vs 3 \n \
        -vs [3,3,5] \n \
        -vs orig \n \
        -vs orig*2 \n \
        -vs orig*4 \n \
        '
    )
    parser.add_argument('-mroi', '--manualroi', action='store_true',
                        help='manual crop before data processing',
                        default=cfg["manualroi"])

    parser.add_argument('-op', '--output_datapath',
                        default=cfg["output_datapath"],
                        help='path for output data')

    parser.add_argument('-ol', '--output_label', default=1,
                        help='label for segmented data')
    parser.add_argument(
        '--slab',
        default=cfg["slab"],
        type=eval,
        help='labels for segmentation,\
            example -slab "{\'liver\':1, \'lesions\':6}"')
    parser.add_argument(
        '-acr', '--autocrop',
        help='automatic crop after data processing',
        default=cfg["autocrop"])
    parser.add_argument(
        '-iparams', '--iparams',
        default=None,
        help='filename of ipars file with stored interactivity')
    parser.add_argument(
        '-sp', '--segparams',
        default=cfg["segparams"],
        help='params for segmentation,\
            example -sp "{\'pairwise_alpha_per_mm2\':90}"')
    parser.add_argument(
        '-tx', '--texture_analysis', action='store_true',
        help='run with texture analysis')
    parser.add_argument('-ed', '--edit_data', action='store_true',
                        help='Run data editor')
    parser.add_argument(
        '-vmax', '--viewermax', type=eval,  # type=int,
        help='Maximum of viewer window, set None for automatic maximum.',
        default=cfg["viewermax"])
    parser.add_argument(
        '-vmin', '--viewermin', type=eval,  # type=int,
        help='Minimum of viewer window, set None for automatic minimum.',
        default=cfg["viewermin"])
    parser.add_argument(
        '--roi', type=eval,  # type=int,
        help='Minimum of viewer window, set None for automatic minimum.',
        default=cfg["roi"])
    parser.add_argument(
        '-so', '--show_output', action='store_true',
        help='Show output data in viewer')
    parser.add_argument(
        '-ni', '--no_interactivity', action='store_true',
        help='run in no interactivity mode, seeds must be defined')
    parser.add_argument('-a', '--arg', nargs='+', type=float)
    parser.add_argument(
        '-ec', '--experiment_caption', type=str,  # type=int,
        help='Short caption of experiment. No special characters.',
        default=cfg["experiment_caption"])
    parser.add_argument(
        '-ids', '--input_datapath_start', type=str,  # type=int,
        help='Start datapath for input dialog.',
        default=cfg["input_datapath_start"])
    parser.add_argument(
        '-oi', '--lisa_operator_identifier', type=str,  # type=int,
        help='Identifier of Lisa operator.',
        default=cfg["lisa_operator_identifier"])
    parser.add_argument(
        '-ss',
        '--segmentation_smoothing',
        action='store_true',
        help='Smoothing of output segmentation',
        default=cfg["segmentation_smoothing"]
    )
    parser.add_argument(
        '--save_filetype', type=str,  # type=int,
        help='File type of saving data. It can be pklz(default), pkl or mat',
        default=cfg["save_filetype"])

    args_obj = parser.parse_args()

    # next two lines brings cfg from file over input parser. This is why there
    # is no need to have cfg param in input arguments
    args = cfg
    args.update(vars(args_obj))
    return args


def boltzman(x, xmid, tau):
    """
    evaluate the boltzman function with midpoint xmid and time constant tau
    over x
    """
    return 1. / (1. + np.exp(-(x-xmid)/tau))

def main():  # pragma: no cover

    #    import ipdb; ipdb.set_trace() # BREAKPOINT
    try:
        ch, fh = logger_init()
        cfg = lisa_config_init()
        args = parser_init(cfg)

        # rint args["arg"]
        oseg_argspec_keys = config.get_function_keys(
            OrganSegmentation.__init__)

        if args["debug"]:
            ch.setLevel(logging.DEBUG)
            args["debug_mode"] = True

        if args["iparams"] is not None:
            params = misc.obj_from_file(args["iparams"], filetype='pickle')

        else:
            params = config.subdict(args, oseg_argspec_keys)

        logger.debug('params ' + str(params))
        oseg = OrganSegmentation(**params)

        if args["no_interactivity"]:
            oseg.ninteractivity()
            oseg.save_outputs()
        else:
            # mport_gui()
            from lisaWindow import OrganSegmentationWindow
            from PyQt4.QtGui import QApplication
            app = QApplication(sys.argv)
            oseg_w = OrganSegmentationWindow(oseg)  # noqa
#    import pdb; pdb.set_trace()
            sys.exit(app.exec_())

    except Exception as e:
        # mport exceptionProcessing
        exceptionProcessing.reportException(e)
        print traceback.format_exc()
        # aise e


if __name__ == "__main__":
    main()
    print "Thank you for using Lisa"



"""
DICTIONNARY INTERFACE FOR EDITING VALUES
creates labels/edits/menubutton widgets in a TkFrame to edit dictionary values
use: apply(frame,dict,position)
"""

import Tkinter as tk

def cbMenu(controlV,value,btn= None):
    controlV.set(str(value))
    if not (btn== None):
        btn.config(text= str(value))

def updateMB(ctrlV, value):
    ctrlV.set(value)

def doLambda(f,*args):
    """Tips: Create lambda within for loop with fixed local variable
    without interference across iterations"""
    def g(): return f(*args)
    return g


def apply(root,d,pos, use_list=True):
    """Creates interface for dictionnary d in root at given grid position """
    "TODO: repercuter kwargs"
    (x,y,w,h)= pos
    lbs= []
    saisies= dict()
    entries= dict()
    for (n,(k,v)) in enumerate(d.iteritems()):
        assert (k not in saisies)
        l= tk.Label(root,text=str(k))
        l.grid(row=n+x,column=y)
        if isinstance(v, list) and use_list:
            """value= list => multiple choice => use menubutton"""
            #saisies[k]= tk.StringVar(name=str(n),value= str(v[0]))
            saisies[k]= tk.StringVar(value= str(v[0]))
            ent=tk.Menubutton(root,textvariable=saisies[k],relief="sunken")
            ent.m=tk.Menu(ent,tearoff=0)
            ent.config(menu=ent.m)
            for (kk,possible) in enumerate(v):
                possibleSaved= "%s" %possible
                ent.m.add_command(label=str(possible), command= doLambda(updateMB,saisies[k],str(d[k][kk]) ) )
                print possible
        else:
            """value is not a list => classical edit => use Entry"""
            #saisies[k]= tk.StringVar(name=str(n),value= str(v))
            saisies[k]= tk.StringVar(value= str(v))
            ent= tk.Entry(textvariable=saisies[k])#,width=30)
        ent.grid(row=n+x,column=y+1)
        entries[k]= ent
    return saisies

def get(strVarDict):
    d= {}
    for (k,v) in strVarDict.iteritems():
        #try: v= float(v)
        #except: pass
        d[k]=v.get()
    return d



def main():
    "EXAMPLE"
    root = tk.Tk()
    #d= {'oui':1, 'non':'non'}
    d= {'oui':1,'a':'b', 'non':['?','!non'],'mode':[1.1,2.1,3.1]}

    v= tk.StringVar(value= "Open File Dialog")

    m=tk.Menubutton(root,textvariable=v,relief="raised")
    m.grid(row=2,column=1)
    mm=tk.Menu(m,tearoff=0)
    tk.Button(root, textvariable=v, command=lambda:v.set('oui')).grid(row=1,column=1)
    mm.add_command(label="go", command=lambda: cbMenu(v,"non"))
    m.config(menu=mm)

    s= apply(root,d,(0,2,0,0))
    print isinstance(d, dict)
    root.mainloop()
    #print d
    print s
    for (k,v) in s.iteritems():
        print str(k), '->',str(v.get())

def testindependance():
    root = tk.Tk()
    d= {'oui':1,'a':'b', 'non':['?','!non'],'mode':[1.1,2.1,3.1]}
    s= apply(root,d,(0,2,0,0))

    dd= {'oui':1,'a':'b', 'non':['?','!non'],'mode':[1.1,2.1,3.1]}
    ss= apply(root,dd,(0,5,0,0))

    print "s =",s
    print "ss=",ss

    print isinstance(d, dict)
    root.mainloop()
    #print d
    #print s
    for (k,v) in s.iteritems():
        print str(k), '->',str(v.get())
    print "-"*10
    for (k,v) in ss.iteritems():
        print str(k), '->',str(v.get())
    print "="*10
    print get(s)
    print get(ss)


if __name__ == '__main__':
    main()
    #testindependance()





#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR%  <>
#
# Distributed under terms of the %LICENSE% license.

"""

"""

import logging

logger = logging.getLogger(__name__)
import argparse
import os.path as op

import config_default
import imtools
import imtools.sample_data
import glob
import os.path as op
import re


def sample_data_path():
    return op.expanduser("~/lisa_data/sample_data/")

def get_sample_data():
    keys = imtools.sample_data.data_urls.keys()
    imtools.sample_data.get_sample_data(keys, sample_data_path())


class DataIteratorOutput:
    def __init__(self, orig, ref, data, label, i):
        self.orig = orig
        self.ref = ref
        self.data = data
        self.label = label
        self.i = i

class DataDirIterator:
    def __init__(
        self,
        data_dir=None,
        reference_dir="~/data/medical/orig/sliver07/training/",
        common_pattern="*[0-9]",
        orig_pattern="*orig%s.mhd",
        seg_pattern="*seg%s.mhd",
        data_pattern="*%s*.pklz"

    ):
        """
        Iterate over all files in reference dir and return file names with defined patter.


        :param data_dir: other related data can be used by defining this parameter
        :param reference_dir: Data directory (f.e. sliver07 training data dir)
        :param common_pattern: used in all other patterns as a kernel. It can select specific files from dir
        :param orig_pattern: select files with orig data
        :param seg_pattern: select files with segmentation
        :param data_pattern: different dir related dir pattern
        """



        self.data_dir = data_dir

        sliver_reference_dir = op.expanduser(reference_dir)


        patterns = seg_pattern % (common_pattern)
        patterno = orig_pattern % (common_pattern)

        self.orig_fnames = glob.glob(sliver_reference_dir + patterno)
        self.ref_fnames = glob.glob(sliver_reference_dir + patterns)
        self.orig_fnames.sort()
        self.ref_fnames.sort()

        if self.data_dir is not None:
            self.data_dir = op.expanduser(data_dir)
            patternd = data_pattern % (common_pattern)
            self.data_fnames = glob.glob(data_dir + patternd)
            self.data_fnames.sort()
        else:
            self.data_fnames = [None] * len(self.orig_fnames)

        self.current = 0
        self.high = len(self.orig_fnames)

    def __iter__(self):
        return self

    def next(self): # Python 3: def __next__(self)
        if self.current >= self.high:
            raise StopIteration
        else:

            numeric_label = re.search(".*g(\d+)", self.orig_fnames[self.current]).group(1)
            out = DataIteratorOutput(
                self.orig_fnames[self.current],
                self.ref_fnames[self.current],
                self.data_fnames[self.current],
                numeric_label,
                self.current

            )
            self.current += 1
            return out


def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    # parser.add_argument(
    #     '-i', '--inputfile',
    #     default=None,
    #     required=True,
    #     help='input file'
    # )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)

    get_sample_data()

if __name__ == "__main__":
    main()


#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/sed3/"))
#sys.path.append(os.path.join(path_to_script, "../extern/"))
#import featurevector
import unittest

import logging
logger = logging.getLogger(__name__)


#import apdb
#  apdb.set_trace();
#import scipy.io
import numpy as np
import scipy
#from scipy import sparse
import traceback

# ----------------- my scripts --------
import sed3
# import dcmreaddata1 as dcmr
# try:
#    import dcmreaddata as dcmr
# except:
#    from pysegbase import dcmreaddata as dcmr

import argparse
#import sed3

import segmentation
import qmisc
import misc
import organ_segmentation
import experiments


def main():

    #logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    # input parser
    params = {'datadir':None, 'working_voxelsize_mm':3}

    dirpath = os.path.join(path_to_script, "../../../data/medical/data_orig/volumetrie")

    experiment_results = {'params':params,'dirpath':dirpath, 'volume_l':{}}

    output_dirpath = os.path.join(path_to_script, '../../../data/medical')
    dirlist = experiments.get_subdirs(dirpath)

    for key in dirlist:
        print key

#        import pdb; pdb.set_trace()
        try:
            dirname = dirlist[key]['abspath']
            params['datadir'] = dirname

            oseg = organ_segmentation.OrganSegmentation(**params)

            oseg.interactivity()
            print (
                    "Volume " +
                    str(oseg.get_segmented_volume_size_mm3() / 1000000.0)
                    + ' [l]')

            volume_l = (oseg.get_segmented_volume_size_mm3() / 1000000.0)

            experiment_results['volume_l'][key] = volume_l

            head, teil = os.path.split(dirname)
            filename_organ = os.path.join(output_dirpath, teil)

            data = oseg.export()
            misc.obj_to_file(data, filename_organ + "-organ.pkl", filetype='pickle')
            #misc.obj_to_file(data, "organ.pkl", filetype='pickle')
            misc.obj_to_file(oseg.get_iparams(), filename_organ + '-iparams.pkl', filetype='pickle')
            misc.obj_to_file(experiment_results, filename_organ + "-info.yaml", filetype='yaml')

        except:
            print 'Selhani, pokracujeme dal'
            print traceback.format_exc()
            import pdb; pdb.set_trace()

    misc.obj_to_file(experiment_results, "results.yaml", filetype='yaml')
    #igc = pycat.ImageGraphCut(data3d, zoom = 0.5)
    #igc.interactivity()

    #igc.make_gc()
    #igc.show_segmentation()

    # volume
    #volume_mm3 = np.sum(oseg.segmentation > 0) * np.prod(oseg.voxelsize_mm)

    #pyed = sed3.sed3(oseg.data3d, contour =
    # oseg.segmentation)
    #pyed.show()

#    if args.show_output:
#        oseg.show_output()
#
#    savestring = raw_input('Save output data? (y/n): ')
#    #sn = int(snstring)
#    if savestring in ['Y', 'y']:
#
#        data = oseg.export()
#
#        misc.obj_to_file(data, "organ.pkl", filetype='pickle')
#        misc.obj_to_file(oseg.get_ipars(), 'ipars.pkl', filetype='pickle')
#    #output = segmentation.vesselSegmentation(oseg.data3d,
    # oseg.orig_segmentation)

if __name__ == "__main__":
    main()
